@@ -10,13 +10,13 @@
 #include "base.h"
 #include "func.h"
 #include "exception.h"
-#include "widgets/XcaApplication.h"
 #include "asn1time.h"
 #include <openssl/x509.h>
 #include <openssl/err.h>
 #include <openssl/opensslv.h>
 
 #include <QObject>
+#include <QLocale>
 
 /* As defined in rfc-5280  4.1.2.5 */
 #define UNDEFINED_DATE "99991231235959Z"
@@ -26,7 +26,7 @@
 
 bool a1time::isUndefined() const
 {
-	return toTime_t() == 0;
+	return toSecsSinceEpoch() == 0;
 }
 
 a1time &a1time::setUndefined()
@@ -34,7 +34,7 @@ a1time &a1time::setUndefined()
 	/* This way we handle "Jan 01, 1970 00:00:00"
 	 * like RFC-5280 undefined date. I dare it */
 	setTimeSpec(Qt::UTC);
-	setTime_t(0);
+	setSecsSinceEpoch(0);
 	return *this;
 }
 
@@ -157,13 +157,13 @@ QString a1time::toString(QString fmt, Qt::TimeSpec spec) const
 		return QObject::tr("Undefined");
 	if (!isValid())
 		return QObject::tr("Broken / Invalid");
-	return XcaApplication::language().toString(
+	return QLocale().toString(
 		spec == Qt::UTC ? toUTC() : toLocalTime(), fmt);
 }
 
 QString a1time::toPretty() const
 {
-	QString fmt = XcaApplication::language().dateTimeFormat();
+	QString fmt = QLocale().dateTimeFormat();
 	return toString(fmt, Qt::LocalTime);
 }
 
@@ -217,7 +217,7 @@ QString a1time::toFancy() const
 		fmt = future ? QObject::tr("in %1 hours") :
 				QObject::tr("%1 hours ago");
 	} else {
-		return XcaApplication::language().toString(date(),
+		return QLocale().toString(date(),
 			QLocale::ShortFormat);
 	}
 	return fmt.arg(diff);
