@@ -9,24 +9,16 @@
 #define __DB_BASE_H
 
 #include <typeinfo>
-#include "db.h"
 #include "base.h"
-#include "load_obj.h"
-#include "widgets/ExportDialog.h"
 #include "pki_base.h"
 #include "headerlist.h"
 
-#include <QListView>
-#include <QPixmap>
-#include <QContextMenuEvent>
 #include <QStringList>
 #include <QAbstractItemModel>
 #include <QHash>
 
 #define X_XCA_DRAG_DATA "application/x-xca-drag-data"
 
-class MainWindow;
-
 class QContextMenuEvent;
 class XcaTreeView;
 class NewX509;
@@ -47,10 +39,6 @@ class db_base: public QAbstractItemModel
 		virtual dbheaderList getHeaders();
 		int colResizing;
 		QString sqlItemSelector();
-		virtual exportType::etype clipboardFormat(QModelIndexList) const
-		{
-			return exportType::Separator;
-		}
 		bool isValidCol(int col) const;
 		void timerEvent(QTimerEvent *event);
 		bool treeview;
@@ -60,7 +48,7 @@ class db_base: public QAbstractItemModel
 	public:
 		void restart_timer();
 		void updateItem(pki_base *pki, const QString &name,
-				const QString &comment);
+				const QString &comment, bool force = false);
 
 		virtual pki_base *newPKI(enum pki_type type = none);
 		db_base(const char *classname);
@@ -75,13 +63,22 @@ class db_base: public QAbstractItemModel
 		virtual void inToCont(pki_base *pki);
 		virtual void remFromCont(const QModelIndex &idx);
 		void changeView();
+		int exportFlags(const QModelIndexList &indexes) const;
+		virtual int exportFlags(const QModelIndex &index) const
+		{
+			return 0;
+		}
+		virtual void exportItem(const QModelIndex &index,
+			const pki_export *xport, XFile &file) const { };
+		virtual void exportItems(const QModelIndexList &indexes,
+			const pki_export *xport, XFile &file) const;
 
-		QPixmap *loadImg(const char *name);
 		void dump(const QString &dirname) const;
-		QModelIndex index(int row, int column, const QModelIndex &parent)const;
-		QModelIndex index(pki_base *pki)const;
+		QModelIndex index(int row, int column,
+				const QModelIndex &parent = QModelIndex()) const;
+		QModelIndex index(pki_base *pki) const;
 		QModelIndex parent(const QModelIndex &index) const;
-		int rowCount(const QModelIndex &parent) const;
+		int rowCount(const QModelIndex &parent = QModelIndex()) const;
 		int allItemsCount() const
 		{
 			return rootItem->childCount();
@@ -104,7 +101,6 @@ class db_base: public QAbstractItemModel
 		{
 			return dynamic_cast<T*>(fromIndex(index));
 		}
-		void load_default(load_base &load);
 		void insertChild(pki_base *child, pki_base *parent = NULL);
 		int rownumber(const pki_base *child) const;
 		void createSuccess(const pki_base *pki) const;
@@ -125,24 +121,19 @@ class db_base: public QAbstractItemModel
 		{
 			colResizing--;
 		}
-		virtual void store(QModelIndexList indexes);
-		virtual void store(QModelIndex index) { (void)index; };
 		dbheaderList getAllHeaders() {
 			return allHeaders;
 		}
-		void pem2clipboard(QModelIndexList indexes) const;
 		QString pem2QString(QModelIndexList indexes) const;
 
 		void deletePKI(QModelIndex idx);
 		QMimeData *mimeData(const QModelIndexList &indexes) const;
-		void editComment(const QModelIndex &index);
 		void emitDataChanged(pki_base *pki);
 		bool containsType(enum pki_type t) const;
 		void writeVcalendar(XFile &file, QStringList vcal) const;
 
 	public slots:
 		virtual void newItem() { }
-		virtual void load() { }
 		void columnResetDefaults();
 		void sectionResized(int i, int, int newSize);
 		void sortIndicatorChanged(int, Qt::SortOrder);
