@@ -8,10 +8,11 @@
 #ifndef __DB_KEY_H
 #define __DB_KEY_H
 
+#include "pki_export.h"
 #include "db_base.h"
 #include "pki_key.h"
+#include "xfile.h"
 
-class MainWindow;
 class QModelIndex;
 class QContextMenuEvent;
 
@@ -21,7 +22,6 @@ class db_key: public db_base
 
 	protected:
 		virtual dbheaderList getHeaders();
-		exportType::etype clipboardFormat(QModelIndexList indexes) const;
 	public:
 		db_key();
 		QList<pki_key*> getUnusedKeys();
@@ -33,10 +33,9 @@ class db_key: public db_base
 		void setOwnPass(QModelIndex idx, enum pki_key::passType);
 		void loadContainer();
 		pki_key *newKey(const keyjob &task, const QString &name);
-
-	public slots:
-		void load();
-		void store(QModelIndex index);
+		int exportFlags(const QModelIndex &index) const;
+		void exportItem(const QModelIndex &index,
+			const pki_export *xport, XFile &file) const;
 
 	signals:
 		void delKey(pki_key *delkey);
