@@ -25,8 +25,8 @@ class db_temp: public db_x509name
 	pki_base *newPKI(enum pki_type type = none);
 	void fillContextMenu(QMenu *menu, const QModelIndex &index);
 	QList<pki_temp*> getPredefs() const;
-	void load();
-	void store(QModelIndex index);
 	bool alterTemp(pki_temp *temp);
+	void exportItem(const QModelIndex &index,
+			const pki_export *, XFile &file) const;
 };
 #endif
