@@ -6,6 +6,8 @@
  */
 
 #include "db_x509.h"
+#include "db_x509req.h"
+#include "db_crl.h"
 #include "pki_x509.h"
 #include "pki_crl.h"
 #include "pki_temp.h"
@@ -17,19 +19,21 @@
 #include "database_model.h"
 #include "entropy.h"
 
-#include "widgets/XcaWarning.h"
+#include "XcaWarningCore.h"
+#include "PwDialogCore.h"
+
+#pragma message ("drop UI dependencies")
+#include "ui_NewX509.h"
 #include "widgets/CertExtend.h"
-#include "widgets/ExportDialog.h"
-#include "widgets/MainWindow.h"
-#include "widgets/PwDialog.h"
 #include "widgets/RevocationList.h"
 #include "widgets/NewX509.h"
-#include "widgets/Help.h"
+#include "widgets/MainWindow.h"
 
-#include "ui_CaProperties.h"
-#include <QMessageBox>
-#include <QContextMenuEvent>
-#include <QAction>
+#include "ui_RevocationList.h"
+#include "ui_MainWindow.h"
+#include "ui_CertExtend.h"
+#include "ui_Revoke.h"
+#include "ui_Help.h"
 
 #include <openssl/rand.h>
 
@@ -60,7 +64,8 @@ void db_x509::loadContainer()
 			qDebug() << "MOVE" << cert->getIntName()
 				<< "from" << cert->getParent()->getIntName()
 				<< "to" << root->getIntName();
-			insertChild(cert, root);
+			treeItem->takeChild(cert);
+			root->insert(cert);
 		}
 	}
 	emit columnsContentChanged();
@@ -118,7 +123,7 @@ void db_x509::remFromCont(const QModelIndex &idx)
 		child = dynamic_cast<pki_x509*>(pki->takeFirst());
 		child->delSigner(dynamic_cast<pki_x509*>(pki));
 		new_parent = child->findIssuer();
-		insertChild(child);
+		insertChild(child, new_parent);
 		if (new_parent)
 			childs << child;
 	}
@@ -152,7 +157,7 @@ static bool recursiveSigning(pki_x509 *cert, pki_x509 *client)
 void db_x509::inToCont(pki_base *pki)
 {
 	pki_x509 *cert = dynamic_cast<pki_x509*>(pki);
-	cert->setParent(NULL);
+	cert->setParent(nullptr);
 	pki_base *root = cert->getSigner();
 
 	insertChild(cert, root);
@@ -188,7 +193,7 @@ void db_x509::inToCont(pki_base *pki)
 		revList.merge(other->getRevList());
 	}
 	/* Search rootItem childs, whether they are ours */
-	foreach(pki_base *b, rootItem->getChildItems()) {
+	foreach(pki_base *b, treeItem->getChildItems()) {
 		pki_x509 *child = dynamic_cast<pki_x509*>(b);
 		if (!child || child == cert || child->getSigner() == child)
 			continue;
@@ -329,12 +334,6 @@ pki_base *db_x509::insert(pki_base *item)
 	return insertPKI(cert);
 }
 
-void db_x509::load(void)
-{
-	load_cert c;
-	load_default(c);
-}
-
 pki_x509 *db_x509::get1SelectedCert()
 {
 	QModelIndexList indexes = mainwin->certView->getSelectedIndexes();
@@ -509,9 +508,8 @@ pki_x509 *db_x509::newCert(NewX509 *dlg)
 			cert->addV3ext(el[i], true);
 	}
 
-	const EVP_MD *hashAlgo = dlg->hashAlgo->currentHash();
 	// and finally sign the request
-	cert->sign(signkey, hashAlgo);
+	cert->sign(signkey, dlg->hashAlgo->current());
 
 	// set the comment field
 	cert->setComment(dlg->comment->toPlainText());
@@ -545,203 +543,93 @@ pki_x509 *db_x509::newCert(NewX509 *dlg)
     return cert;
 }
 
-void db_x509::store(QModelIndex idx)
-{
-	QModelIndexList l;
-	l << idx;
-	store(l);
-}
-
-void db_x509::store(QModelIndexList list)
+int db_x509::exportFlags(const QModelIndex &idx) const
 {
 	QStringList filt;
-	bool chain;
-	QList<exportType> types, usual;
+	int disable_flags = 0;
 
-	if (list.size() == 0)
-		return;
-
-	pki_x509 *oldcrt, *crt = fromIndex<pki_x509>(list[0]);
+	pki_x509 *crt = fromIndex<pki_x509>(idx);
 	if (!crt)
-		return;
+		return 0;
 
 	pki_key *privkey = crt->getRefKey();
-	pki_evp *pkey;
-	chain = crt->getSigner() && crt->getSigner() != crt;
 
-	usual <<
-		exportType(exportType::PEM, "crt", "PEM") <<
-		exportType(exportType::PKCS7, "p7b", "PKCS #7");
+	if (!crt->getSigner() || crt->getSigner() == crt)
+		disable_flags |= F_CHAIN;
 
-	types << exportType(exportType::DER, "cer", "DER");
+	if (!privkey || !privkey->isPrivKey() || privkey->isToken())
+		disable_flags |= F_PRIVATE;
 
-	if (list.size() > 1) {
-		usual <<
-			exportType(exportType::PEM_selected, "pem",
-				"PEM selected") <<
-			exportType(exportType::PKCS7_selected, "pem",
-				"PKCS7 selected");
-	}
-	if (chain) {
-		types <<
-			exportType(exportType::PEM_chain, "pem",
-				tr("PEM chain")) <<
-			exportType(exportType::PKCS7_chain, "p7b",
-				tr("PKCS#7 chain"));
-	}
+	if (!crt->isCA())
+		disable_flags |= F_CA;
 
-	if (privkey && privkey->isPrivKey() && !privkey->isToken()) {
-		if (chain) {
-			usual << exportType(exportType::PKCS12_chain, "pfx",
-				tr("PKCS#12 chain"));
-			types << exportType(exportType::PKCS12, "pfx",
-				"PKCS #12");
-		} else {
-			usual << exportType(exportType::PKCS12, "pfx",
-				"PKCS #12");
-		}
-		types <<
-			exportType(exportType::PEM_cert_key, "pem",
-				tr("PEM + key")) <<
-			exportType(exportType::PEM_cert_pk8, "pem",
-				"PEM + PKCS#8");
-	}
-	types << exportType() <<
-		exportType(exportType::PKCS7_unrevoked, "p7b",
-			tr("PKCS#7 unrevoked")) <<
-		exportType(exportType::PKCS7_all, "p7b",
-			tr("PKCS#7 all")) <<
-		exportType(exportType::PEM_unrevoked, "pem",
-			tr("PEM unrevoked")) <<
-		exportType(exportType::PEM_all, "pem",
-			tr("PEM all")) <<
-		exportType(exportType::Index, "txt",
-			tr("Certificate Index file"));
-	if (crt->getNotAfter() > a1time())
-		types << exportType(exportType::vcalendar, "ics",
-			tr("vCalendar"));
+	return disable_flags;
+}
 
-	if (crt->isCA())
-		types << exportType(exportType::vcalendar_ca, "ics",
-			tr("CA vCalendar"));
-
-	types = usual << exportType() << types;
-	ExportDialog *dlg = new ExportDialog(NULL, tr("Certificate export"),
-		tr("X509 Certificates ( *.pem *.cer *.crt *.p12 *.pfx *.p7b )"), crt,
-		QPixmap(":certImg"), types, "certexport");
-	if (!dlg->exec()) {
-		delete dlg;
+void db_x509::exportItems(const QModelIndexList &list,
+			const pki_export *xport, XFile &file) const
+{
+	if (list.empty())
 		return;
-	}
-	QStringList vcal;
+
+	pki_x509 *oldcrt = nullptr, *crt = fromIndex<pki_x509>(list[0]);
+
 	QList<pki_x509*> certs;
-	QList<pki_base*> items;
-	enum exportType::etype type = dlg->type();
-	try {
-		XFile file(dlg->filename->text());
-		file.open_write();
-		pki_base::pem_comment = dlg->pemComment->isChecked();
+	foreach(QModelIndex idx, list) {
+		pki_x509 *x = fromIndex<pki_x509>(idx);
+		if (x)
+			certs << x;
+	}
 
-		switch (type) {
-		case exportType::PEM:
+	if (xport->match_all(F_PEM | F_CHAIN)) {
+		while (crt && crt != oldcrt) {
 			crt->writeCert(file, true);
-			break;
-		case exportType::PEM_chain:
-			while (crt && crt != oldcrt) {
-				crt->writeCert(file, true);
-				oldcrt = crt;
-				crt = crt->getSigner();
-			}
-			break;
-		case exportType::PEM_selected:
-			foreach(QModelIndex idx, list) {
-				crt = fromIndex<pki_x509>(idx);
-				if (crt)
-					crt->writeCert(file, true);
-			}
-			break;
-		case exportType::PEM_unrevoked:
-			foreach(pki_x509 *pki, Store.getAll<pki_x509>()) {
-				if (!pki->isRevoked())
-					pki->writeCert(file, true);
-			}
-			break;
-		case exportType::PEM_all:
-			foreach(pki_x509 *pki, Store.getAll<pki_x509>()) {
-				pki->writeCert(file, true);
-			}
-			break;
-		case exportType::DER:
-			crt->writeCert(file, false);
-			break;
-		case exportType::PKCS7:
-		case exportType::PKCS7_chain:
-		case exportType::PKCS7_unrevoked:
-		case exportType::PKCS7_selected:
-		case exportType::PKCS7_all:
-			writePKCS7(crt, file, type, list);
-			break;
-		case exportType::PKCS12:
-			writePKCS12(crt, file, false);
-			break;
-		case exportType::PKCS12_chain:
-			writePKCS12(crt, file, true);
-			break;
-		case exportType::PEM_cert_pk8:
-		case exportType::PEM_cert_key:
-			pkey = (pki_evp *)crt->getRefKey();
-			if (!pkey || pkey->isPubKey()) {
-				XCA_WARN(tr("There was no key found for the Certificate: '%1'").
-					arg(crt->getIntName()));
-				break;
-			}
-			if (pkey->isToken()) {
-				XCA_WARN(tr("Not possible for a token key: '%1'").
-					arg(crt->getIntName()));
-				break;
-			}
-
-			if (type == exportType::PEM_cert_pk8) {
-				pkey->writePKCS8(file, EVP_des_ede3_cbc(),
-						PwDialog::pwCallback, true);
-			} else {
-				pkey->writeKey(file, NULL, NULL, true);
-			}
-			crt->writeCert(file, true);
-			break;
-		case exportType::Index:
-			foreach(QModelIndex idx, list) {
-				crt = fromIndex<pki_x509>(idx);
-				if (crt)
-					certs << crt;
-			}
-			writeIndex(file, certs);
-			break;
-		case exportType::vcalendar:
-			foreach(QModelIndex idx, list) {
-				crt = fromIndex<pki_x509>(idx);
-				if (crt)
-					vcal += crt->icsVEVENT();
-			}
-			writeVcalendar(file, vcal);
-			break;
-		case exportType::vcalendar_ca:
-			foreach(QModelIndex idx, list) {
-				crt = fromIndex<pki_x509>(idx);
-				if (crt)
-					vcal += crt->icsVEVENT_ca();
-			}
-			writeVcalendar(file, vcal);
-			break;
-		default:
-			exit(1);
+			oldcrt = crt;
+			crt = crt->getSigner();
 		}
+	} else if (xport->match_all(F_PEM)) {
+		foreach(crt, certs)
+			crt->writeCert(file, true);
+	} else if (xport->match_all(F_PEM | F_UNREVOKED)) {
+		foreach(pki_x509 *pki, Store.getAll<pki_x509>())
+			if (!pki->isRevoked())
+				pki->writeCert(file, true);
+	} else if (xport->match_all(F_PEM | F_ALL)) {
+		foreach(pki_x509 *pki, Store.getAll<pki_x509>())
+			pki->writeCert(file, true);
+	} else if (xport->match_all(F_PEM | F_PRIVATE)) {
+		pki_evp *pkey = (pki_evp *)crt->getRefKey();
+		if (!pkey || pkey->isPubKey())
+			throw errorEx(tr("There was no key found for the Certificate: '%1'").
+				arg(crt->getIntName()));
+		if (pkey->isToken())
+			throw errorEx(tr("Not possible for a token key: '%1'").
+				arg(crt->getIntName()));
+		if (xport->match_all(F_PKCS8)) {
+			pkey->writePKCS8(file, EVP_des_ede3_cbc(),
+				PwDialogCore::pwCallback, true);
+		} else {
+			pkey->writeKey(file, NULL, NULL, true);
+		}
+		crt->writeCert(file, true);
+	} else if (xport->match_all(F_PEM)) {
+		crt->writeCert(file, true);
+	} else if (xport->match_all(F_DER)) {
+		crt->writeCert(file, false);
+	} else if (xport->match_all(F_PKCS7)) {
+		writePKCS7(crt, file, xport->flags, list);
+	} else if (xport->match_all(F_PKCS12)) {
+		writePKCS12(crt, file, xport->match_all(F_CHAIN));
+	} else if (xport->match_all(F_INDEX)) {
+		writeIndex(file, certs);
+	} else if (xport->match_all(F_CAL)) {
+		QStringList vcal;
+		foreach(crt, certs) {
+			vcal += xport->match_all(F_CHAIN) ?
+				crt->icsVEVENT_ca() : crt->icsVEVENT();
+		}
+		writeVcalendar(file, vcal);
 	}
-	catch (errorEx &err) {
-		XCA_ERROR(err);
-	}
-	pki_base::pem_comment = false;
-	delete dlg;
 }
 
 void db_x509::writeIndex(XFile &file, QList<pki_x509*> items) const
@@ -775,7 +663,8 @@ void db_x509::writePKCS12(pki_x509 *cert, XFile &file, bool chain) const
 			cert = signer;
 			signer = signer->getSigner();
 		}
-		p12->writePKCS12(file);
+		encAlgo encAlgo((QString) Settings["pkcs12_enc_algo"]);
+		p12->writePKCS12(file, encAlgo);
 	}
 	catch (errorEx &err) {
 		XCA_ERROR(err);
@@ -783,43 +672,32 @@ void db_x509::writePKCS12(pki_x509 *cert, XFile &file, bool chain) const
 	delete p12;
 }
 
-void db_x509::writePKCS7(pki_x509 *cert, XFile &file, exportType::etype type,
-			QModelIndexList list) const
+void db_x509::writePKCS7(pki_x509 *cert, XFile &file, int flags,
+			const QModelIndexList &list) const
 {
-	pki_pkcs7 *p7 = NULL;
+	pki_pkcs7 *p7 = new pki_pkcs7(QString());
 
 	try {
-		p7 = new pki_pkcs7(QString());
-		switch (type) {
-		case exportType::PKCS7_chain:
-			while (cert != NULL) {
+		if (flags & F_CHAIN) {
+			while (cert) {
 				p7->append_item(cert);
 				if (cert->getSigner() == cert)
-					cert = NULL;
-				else
-					cert = cert->getSigner();
+					break;
+				cert = cert->getSigner();
 			}
-			break;
-		case exportType::PKCS7:
-			p7->append_item(cert);
-			break;
-		case exportType::PKCS7_selected:
+		} else if (flags & (F_UNREVOKED | F_ALL)) {
+			foreach(pki_x509 *cer, Store.getAll<pki_x509>()) {
+				if ((flags & F_ALL) || !cer->isRevoked())
+					p7->append_item(cer);
+			}
+		} else if (flags) {
 			foreach(QModelIndex idx, list) {
 				cert = fromIndex<pki_x509>(idx);
 				if (cert)
 					p7->append_item(cert);
 			}
-			break;
-		case exportType::PKCS7_unrevoked:
-		case exportType::PKCS7_all:
-			foreach(pki_x509 *cer, Store.getAll<pki_x509>()) {
-				if ((type == exportType::PKCS7_all) ||
-				    (!cer->isRevoked()))
-					p7->append_item(cer);
-			}
-			break;
-		default:
-			exit(1);
+		} else {
+			p7->append_item(cert);
 		}
 		p7->writeP7(file, false);
 	}
@@ -838,6 +716,7 @@ void db_x509::certRenewal(QModelIndexList indexes)
 	CertExtend *dlg = NULL;
 	x509rev r;
 	bool doRevoke = false;
+	bool doReplace = false;
 
 	if (indexes.size() == 0)
 		return;
@@ -863,6 +742,7 @@ void db_x509::certRenewal(QModelIndexList indexes)
 			r = revoke->getRevocation();
 			delete revoke;
 		}
+		doReplace = dlg->replace->isChecked();
 		foreach(idx, indexes) {
 			oldcert = fromIndex<pki_x509>(idx);
 			if (!oldcert)
@@ -888,6 +768,10 @@ void db_x509::certRenewal(QModelIndexList indexes)
 			newcert->sign(signkey, oldcert->getDigest());
 			newcert = dynamic_cast<pki_x509 *>(insert(newcert));
 			createSuccess(newcert);
+
+			// delete old certificate if requested
+			if (doReplace)
+				deletePKI(idx);
 		}
 		if (doRevoke)
 			do_revoke(indexes, r);
@@ -1007,7 +891,7 @@ void db_x509::toRequest(QModelIndex idx)
 
 	try {
 		pki_x509req *req = new pki_x509req();
-		check_oom(req);
+		Q_CHECK_PTR(req);
 		req->pkiSource = transformed;
 		req->setIntName(cert->getIntName());
 		req->createReq(cert->getRefKey(), cert->getSubject(),
@@ -1031,56 +915,21 @@ void db_x509::toToken(QModelIndex idx, bool alwaysSelect)
         }
 }
 
-void db_x509::caProperties(QModelIndex idx)
+void db_x509::updateCaProperties(pki_x509 *cert)
 {
-	QStringList actions;
-	Ui::CaProperties ui;
+	XSqlQuery q;
+	Transaction;
+	TransThrow();
 
-	pki_x509 *cert = fromIndex<pki_x509>(idx);
-	if (!cert)
-		return;
+	SQL_PREPARE(q, "UPDATE authority SET crlDays=?, "
+			"template=? WHERE item=?");
 
-	QDialog *dlg = new QDialog(NULL);
-	ui.setupUi(dlg);
-	ui.days->setSuffix(QString(" ") + tr("days"));
-	ui.days->setMaximum(1000000);
-	ui.days->setValue(cert->getCrlDays());
-	ui.image->setPixmap(QPixmap(":certImg"));
+	q.bindValue(0, cert->getCrlDays());
+	q.bindValue(1, cert->getTemplateSqlId());
+	q.bindValue(2, cert->getSqlItemId());
 
-	QVariant tmplId = cert->getTemplateSqlId();
-	pki_temp *templ = Store.lookupPki<pki_temp>(tmplId);
-
-	ui.temp->insertPkiItems(Store.getAll<pki_temp>());
-        ui.temp->setNullItem(tr("No template"));
-	ui.temp->setCurrentIndex(0);
-	if (templ)
-		ui.temp->setCurrentPkiItem(templ);
-
-	ui.certName->setTitle(cert->getIntName());
-	mainwin->helpdlg->register_ctxhelp_button(dlg, "ca_properties");
-
-	if (dlg->exec()) {
-		XSqlQuery q;
-		QSqlError e;
-		Transaction;
-		TransThrow();
-
-		templ = ui.temp->currentPkiItem();
-		tmplId = templ ? templ->getSqlItemId() : QVariant();
-
-		cert->setTemplateSqlId(tmplId);
-		cert->setCrlDays(ui.days->value());
-
-		SQL_PREPARE(q, "UPDATE authority SET crlDays=?, "
-				"template=? WHERE item=?");
-
-		q.bindValue(0, cert->getCrlDays());
-		q.bindValue(1, tmplId);
-		q.bindValue(2, cert->getSqlItemId());
-		AffectedItems(cert->getSqlItemId());
-		q.exec();
-	        TransDone(q.lastError());
-		XCA_SQLERROR(q.lastError());
-	}
-	delete dlg;
+	AffectedItems(cert->getSqlItemId());
+	q.exec();
+	TransDone(q.lastError());
+	XCA_SQLERROR(q.lastError());
 }
