@@ -9,6 +9,7 @@
 #ifndef __DB_X509_H
 #define __DB_X509_H
 
+#include "pki_export.h"
 #include "db_x509super.h"
 #include "asn1int.h"
 #include "x509rev.h"
@@ -46,22 +47,22 @@ class db_x509: public db_x509super
 		pki_x509 *newCert(NewX509 *dlg);
 		void newCert(pki_x509 *cert);
 		void writePKCS12(pki_x509 *cert, XFile &file, bool chain) const;
-		void writePKCS7(pki_x509 *cert, XFile &file,
-			exportType::etype type, QModelIndexList list) const;
+		void writePKCS7(pki_x509 *cert, XFile &file, int flags,
+				const QModelIndexList &list) const;
 		void fillContextMenu(QMenu *menu, const QModelIndex &index);
 		void inToCont(pki_base *pki);
 		a1int getUniqueSerial(pki_x509 *signer);
 		void toToken(QModelIndex idx, bool alwaysSelect);
 		void toRequest(QModelIndex idx);
-		void store(QModelIndex idx);
-		void store(QModelIndexList list);
-		void load();
-		void caProperties(QModelIndex idx);
+		void updateCaProperties(pki_x509 *cert);
 		void toCertificate(QModelIndex index);
 		void certRenewal(QModelIndexList indexes);
 		void revoke(QModelIndexList indexes);
 		void do_revoke(QModelIndexList indexes, const x509rev &r);
 		void unRevoke(QModelIndexList indexes);
+		int exportFlags(const QModelIndex &idx) const;
+		void exportItems(const QModelIndexList &indexes,
+				const pki_export *xport, XFile &file) const;
 
 	public slots:
 		void newItem();
