@@ -8,7 +8,6 @@
 #include <QString>
 #include <QDir>
 
-#include "widgets/OpenDb.h"
 #include "xfile.h"
 #include "func.h"
 #include "dbhistory.h"
@@ -18,6 +17,7 @@ static QString dbhistory_file()
 {
 	return getUserSettingsDir() +  "/dbhistory";
 }
+QString dbhistory::lastRemote;
 
 dbhistory::dbhistory()
 {
@@ -32,11 +32,11 @@ dbhistory::dbhistory()
 
 	history.clear();
 	while (!file.atEnd()) {
-		char buf[1024];
-		ssize_t size = file.readLine(buf, sizeof buf);
-		if (size <= 0)
+		QByteArray ba;
+		ba = file.readLine(1024);
+		if (ba.size() == 0)
 			break;
-		name = QString::fromUtf8(buf).trimmed();
+		name = QString::fromUtf8(ba).trimmed();
 		if (name.size() == 0)
 			continue;
 		if (history.indexOf(name) == -1)
@@ -46,7 +46,7 @@ dbhistory::dbhistory()
 
 	foreach(name, history) {
 		if (database_model::isRemoteDB(name)) {
-			OpenDb::setLastRemote(name);
+			setLastRemote(name);
 			break;
 		}
 	}
@@ -82,3 +82,13 @@ void dbhistory::addEntry(const QString &name)
 	file.close();
 }
 
+void dbhistory::setLastRemote(const QString &db)
+{
+	if (database_model::isRemoteDB(db))
+		lastRemote = db;
+}
+
+QString dbhistory::getLastRemote()
+{
+	return lastRemote;
+}
