@@ -11,8 +11,9 @@
 #include <QDebug>
 
 #include "widgets/MainWindow.h"
+#include "ui_MainWindow.h"
 #include "widgets/XcaApplication.h"
-#include "widgets/XcaWarning.h"
+#include "XcaWarningCore.h"
 #include "func.h"
 #include "xfile.h"
 #include "main.h"
@@ -23,17 +24,20 @@
 #include "pki_evp.h"
 #include "pki_base.h"
 #include "arguments.h"
+#include "pki_export.h"
 #include "db_x509.h"
 #if defined(Q_OS_WIN32)
 //For the segfault handler
 #include <windows.h>
 #endif
 
+#include <QTextStream>
+
+void migrateOldPaths();
+
 char segv_data[1024];
 MainWindow *mainwin = NULL;
 
-static int debug;
-
 #if defined(Q_OS_WIN32)
 static LONG CALLBACK w32_segfault(LPEXCEPTION_POINTERS e)
 {
@@ -55,7 +59,139 @@ static void segv_handler_gui(int)
 }
 #endif
 
-void myMsgOutput(QtMsgType type, const char *msg)
+class dbg_pattern
+{
+		QString file, func;
+		unsigned first, last;
+		bool inv;
+	public:
+		bool invert() const { return inv; }
+		dbg_pattern(QString);
+		bool match(const QString &curr_file, const QString &curr_func,
+					int line) const;
+};
+
+class debug_info
+{
+	private:
+		QString short_file;
+		QString short_func;
+		int line;
+
+		static QList<dbg_pattern> patternlist;
+	public:
+		static bool all;
+		static void set_debug(const QString &dbg);
+		debug_info(const QMessageLogContext &c);
+		QString log_prefix() const;
+		bool do_debug() const;
+		static bool isEmpty()
+		{
+			return patternlist.size() == 0;
+		}
+};
+
+QList<dbg_pattern> debug_info::patternlist;
+bool debug_info::all = false;
+
+dbg_pattern::dbg_pattern(QString part)
+	: first(0), last(INT_MAX), inv(false)
+{
+	bool ok;
+	if (part[0] == '-') {
+		inv = true;
+		part.remove(0, 1);
+	}
+	file = func = part;
+	QStringList file_num = part.split(":");
+	if (file_num.size() == 2) {
+		file = file_num[0];
+		file_num = file_num[1].split("-");
+		if (file_num.size() == 1) {
+			first = last = file_num[0].toUInt();
+		} else {
+			if (!file_num[0].isEmpty()) {
+				first = file_num[0].toUInt(&ok);
+				Q_ASSERT(ok);
+			}
+			if (!file_num[1].isEmpty()) {
+				last = file_num[1].toUInt(&ok);
+				Q_ASSERT(ok);
+			}
+		}
+	}
+	qDebug() << "New debug match" << (inv ? "Not" : "") << file << func << first << last;
+}
+
+bool dbg_pattern::match(const QString &curr_file, const QString &curr_func,
+						int line) const
+{
+	// QTextStream out(stdout);
+	// out << QString("MATCH %1:%2(%3)\n").arg(curr_file).arg(curr_func).arg(line);
+	if (curr_func == func)
+		return true;
+	if (curr_func.endsWith(QString("::%1").arg(func)))
+		return true;
+	if (curr_file != file && !file.endsWith(QString("/%1").arg(curr_file)))
+		return false;
+	if (line >= first && line <= last)
+		return true;
+	return false;
+}
+
+void debug_info::set_debug(const QString &dbg)
+{
+	bool local_all = false;
+	all = true;
+	if (isEmpty()) {
+		foreach(QString part, dbg.split(",")) {
+			if (part.toLower() == "all") {
+				local_all = true;
+				continue;
+			}
+			dbg_pattern d(part);
+			patternlist.insert(d.invert() ? 0 : patternlist.size(), d);
+		}
+	}
+	all = local_all;
+}
+
+debug_info::debug_info(const QMessageLogContext &ctx)
+	: line(0)
+{
+	line = ctx.line;
+	if (ctx.file && ctx.line) {
+		int pos;
+		short_file = ctx.file, short_func = ctx.function;
+		pos = short_file.lastIndexOf("/");
+		short_file.remove(0, pos +1);
+		pos = short_func.indexOf("(");
+		short_func.remove(pos, short_func.size());
+		pos = short_func.lastIndexOf(" ");
+		short_func.remove(0, pos +1);
+	}
+	//std::cerr << "DBG '" << (ctx.function ?: "(NULL)" )<< "' '" << CCHAR(short_func) << "' " << std::endl;
+}
+
+QString debug_info::log_prefix() const
+{
+	if (short_file == nullptr && line == 0)
+		return QString();
+	return QString(" " COL_MAGENTA "%1" COL_GREEN COL_BOLD ":%2 " COL_BLUE "%3")
+					.arg(short_file).arg(line).arg(short_func);
+}
+
+bool debug_info::do_debug() const
+{
+	foreach(dbg_pattern pattern, patternlist) {
+		if (pattern.match(short_file, short_func, line))
+			return !pattern.invert();
+	}
+	return all;
+}
+
+void myMessageOutput(QtMsgType type, const QMessageLogContext &ctx,
+			const QString &msg)
 {
 	static QElapsedTimer *t;
 	static int abort_on_warning = -1;
@@ -63,47 +199,38 @@ void myMsgOutput(QtMsgType type, const char *msg)
 	int el;
 
 	if (!t) {
-		char *d = getenv("XCA_DEBUG");
 		t = new QElapsedTimer();
 		t->start();
-		if (d && *d)
-			debug = 1;
 	}
 	if (abort_on_warning == -1) {
 		char *a = getenv("XCA_ABORT_ON_WARNING");
 		abort_on_warning = a && *a;
 	}
+	debug_info dinfo(ctx);
 	el = t->elapsed();
 	switch (type) {
 	case QtDebugMsg:
-		if (!debug)
+		if (!dinfo.do_debug())
 			return;
 		severity = COL_CYAN "Debug";
 		break;
 	case QtWarningMsg:  warn_msg = "WARNING";  severity = COL_LRED "Warning"; break;
 	case QtCriticalMsg: warn_msg = "CRITICAL"; severity = COL_RED "Critical"; break;
 	case QtFatalMsg:    warn_msg = "FATAL";    severity = COL_RED "Fatal"; break;
-#if QT_VERSION >= 0x050000
 	case QtInfoMsg:	    severity = COL_CYAN "Info"; break;
-#endif
 	default:            severity = COL_CYAN "Default"; break;
 	}
-	console_write(stderr, QString(COL_YELL "%1%2 %3:" COL_RESET " %4\n")
+	console_write(stderr, QString(COL_YELL "%1%2 %3:%5" COL_RESET " %4\n")
 			.arg(el/1000, 4)
 			.arg((el%1000)/100, 2, 10, QChar('0'))
-			.arg(severity).arg(QString::fromUtf8(msg)).toUtf8());
+			.arg(severity).arg(msg)
+			.arg(dinfo.log_prefix()).toUtf8());
 
 	if (abort_on_warning == 1 && warn_msg) {
 		qFatal("Abort on %s", warn_msg);
 	}
 }
 
-#if QT_VERSION >= 0x050000
-void myMessageOutput(QtMsgType t, const QMessageLogContext &, const QString &m)
-{
-	myMsgOutput(t, m.toUtf8().constData());
-}
-#endif
 
 static void cmd_version(FILE *fp)
 {
@@ -151,18 +278,25 @@ static Passwd acquire_password(QString source)
 	return pass;
 }
 
+static bool compare_pki_base(pki_base* a, pki_base* b)
+{
+	return (a->getSqlItemId().toULongLong() <
+		b->getSqlItemId().toULongLong());
+}
+
 static pki_multi *cmdline_items;
-static void read_cmdline(int argc, char *argv[])
+static void read_cmdline(int argc, char *argv[], bool console_only)
 {
 	arguments cmd_opts(argc, argv);
 	pki_evp::passwd = acquire_password(cmd_opts["password"]);
 	Passwd sqlpw = acquire_password(cmd_opts["sqlpass"]);
 
-	if (cmd_opts.has("verbose"))
-		debug = 1;
-
-	if (cmd_opts.getResult() != 0)
-		cmd_help(EXIT_FAILURE, cmd_opts.resultString().toUtf8());
+	if (cmd_opts.has("verbose")) {
+		QString all = cmd_opts["verbose"];
+		debug_info::set_debug(all.isEmpty() ? QString("all") : all);
+	}
+	if (!cmd_opts.has("password") && console_only)
+		database_model::open_without_password = true;
 
 	if (cmd_opts.has("database"))
 		Database.open(cmd_opts["database"], sqlpw);
@@ -173,6 +307,14 @@ static void read_cmdline(int argc, char *argv[])
 		qDebug() << "Probe" << file;
 		cmdline_items->probeAnything(file);
 	}
+	QStringList names = cmd_opts["import-names"].split(";");
+	foreach(pki_base *pki, cmdline_items->get()) {
+		if (names.isEmpty())
+			break;
+		QString name = names.takeFirst();
+		if (!name.isEmpty())
+			pki->setIntName(name);
+	}
 	if (cmdline_items->failed_files.size() > 0) {
 		XCA_WARN(QString("Failed to import from '%1'")
 			.arg(cmdline_items->failed_files.join("' '")));
@@ -194,6 +336,8 @@ static void read_cmdline(int argc, char *argv[])
 			cmd_help(EXIT_FAILURE, msg[opt]);
 		}
 	}
+	database_model::open_without_password = false;
+
 	if (cmd_opts.has("list-curves")) {
 		QStringList list;
 		foreach(const builtin_curve &c, builtinCurves) {
@@ -203,6 +347,19 @@ static void read_cmdline(int argc, char *argv[])
 		}
 		console_write(stdout, list.join("\n").toUtf8() + '\n');
 	}
+	if (cmd_opts.has("list-items")) {
+		QStringList list;
+		QList<pki_base*> items = Store.getAll<pki_base>();
+		std::sort(items.begin(), items.end(), compare_pki_base);
+		foreach(pki_base *pki, items) {
+			list << QString(COL_YELL "%1 " COL_GREEN "%2 "
+					COL_RESET "%3")
+					.arg(pki->getSqlItemId().toString(), 7)
+					.arg(pki->getTypeString(), -27)
+					.arg(pki->getIntName());
+		}
+		console_write(stdout, list.join("\n").toUtf8() + '\n');
+	}
 	if (!cmd_opts["index"].isEmpty()) {
 		qDebug() << cmd_opts["index"];
 		db_x509 *certs = Database.model<db_x509>();
@@ -239,13 +396,15 @@ static void read_cmdline(int argc, char *argv[])
 		QStringList out;
 		db_x509 *certs = Database.model<db_x509>();
 		QList<pki_x509*>issuers = certs->getAllIssuers();
+		std::sort(issuers.begin(), issuers.end(), compare_pki_base);
 		foreach(pki_x509 *iss, issuers) {
 			pki_key *key = iss->getRefKey();
 			QString keytype = key ? key->getTypeString() : "";
-			out << QString("%1 '%2' %3")
-				.arg(iss->getSqlItemId().toULongLong(), 4)
-				.arg(iss->getIntName())
-				.arg(keytype);
+			out << QString(COL_YELL "%1 " COL_GREEN "%2 "
+					COL_RESET "%3")
+				.arg(iss->getSqlItemId().toULongLong(), 7)
+				.arg(keytype, -13)
+				.arg(iss->getIntName());
 		}
 		console_write(stdout, out.join("\n").toUtf8() + '\n');
 	}
@@ -268,11 +427,21 @@ static void read_cmdline(int argc, char *argv[])
 					.arg(cmd_opts["crlgen"]));
 		} else {
 			crljob task(issuer);
-			pki_crl *crl = crls->newCrl(task);
+			pki_crl *crl = crls->newCrl(task, cmd_opts["name"]);
 			if (crl)
 				cmdline_items->append_item(crl);
 		}
 	}
+	if (!cmd_opts["select"].isEmpty()) {
+		foreach(QString item, cmd_opts["select"].split(",")) {
+			bool ok;
+			qDebug() << "Select" << item;
+			qulonglong id = item.toULongLong(&ok);
+			pki_base *pki = Store.lookupPki<pki_base>(QVariant(id));
+			if (pki)
+				cmdline_items->append_item(pki);
+		}
+	}
 
 	BioByteArray bba;
 	foreach(pki_base *pki, cmdline_items->get()) {
@@ -300,17 +469,23 @@ static void read_cmdline(int argc, char *argv[])
 
 int main(int argc, char *argv[])
 {
-	const char *xca_special = getenv("XCA_SPECIAL");
+	const char *xca_special = getenv("XCA_ARGUMENTS");
 	if (xca_special && *xca_special) {
 		puts(CCHAR(arguments::doc(xca_special)));
 		return 0;
 	}
-#if QT_VERSION < 0x050000
-	qInstallMsgHandler(myMsgOutput);
-#else
 	qInstallMessageHandler(myMessageOutput);
-#endif
+	{
+		const char *d = getenv("XCA_DEBUG");
+		if (d && *d)
+			debug_info::set_debug(QString(d));
+	}
+
 #if defined(Q_OS_WIN32)
+	// If no style provided externally
+	if (!QApplication::style())
+		QApplication::setStyle("Fusion");
+
 	AttachConsole(-1);
 
 	int wargc;
@@ -339,25 +514,34 @@ int main(int argc, char *argv[])
 		xca_name = argv[0];
 
 	bool console_only = arguments::is_console(argc, argv);
-	XcaApplication *gui;
+	XcaApplication *gui = nullptr;
+	QCoreApplication *coreApp = nullptr;
 
 #if !defined(Q_OS_WIN32)
 	if (console_only) {
-		new QCoreApplication(argc, argv);
-		gui = NULL;
+		coreApp = new QCoreApplication(argc, argv);
 	} else
 #endif
 	{
 		/* On windows, always instantiate a GUI app */
-		gui = new XcaApplication(argc, argv);
+		coreApp = gui = new XcaApplication(argc, argv);
+		is_gui_app = true;
 	}
 
-	if (!QDir().mkpath(getUserSettingsDir()))
-		qCritical("Failed to create Path: '%s'", CCHAR(getUserSettingsDir()));
+	coreApp->setApplicationName(PACKAGE_TARNAME);
+	coreApp->setOrganizationDomain("de.hohnstaedt");
+	coreApp->setApplicationVersion(XCA_VERSION);
+	xcaWarning::setGui(new xcaWarningCore());
+
+	migrateOldPaths();
 
 	Entropy entropy;
 	Settings.clear();
-	initOIDs();
+	try {
+		initOIDs();
+	} catch (errorEx &e) {
+		XCA_ERROR(e);
+	}
 
 	for (int i=0; i < argc; i++)
 		qDebug() << "wargv" << argc << i << argv[i];
@@ -365,17 +549,21 @@ int main(int argc, char *argv[])
 		if (gui && !console_only) {
 			mainwin = new MainWindow();
 			gui->setMainwin(mainwin);
-			read_cmdline(argc, argv);
+			read_cmdline(argc, argv, console_only);
 			mainwin->importMulti(cmdline_items, 1);
 			cmdline_items = NULL;
+			enum open_result r = open_abort;
 			if (!Database.isOpen())
-				mainwin->init_database(QString());
+				r = mainwin->init_database(QString());
 			else
-				mainwin->setup_open_database();
-			mainwin->show();
-			gui->exec();
+				r = mainwin->setup_open_database();
+			qDebug() << "PWret" << r << pw_cancel << pw_ok;
+			if (r != pw_exit) {
+				mainwin->show();
+				gui->exec();
+			}
 		} else {
-			read_cmdline(argc, argv);
+			read_cmdline(argc, argv, console_only);
 			delete cmdline_items;
 		}
 	} catch (errorEx &ex) {
@@ -391,6 +579,7 @@ int main(int argc, char *argv[])
 			 << pki->getIntName();
 	delete mainwin;
 	delete gui;
+	pki_export::free_elements();
 #if defined(Q_OS_WIN32)
 	FreeConsole();
 #endif
