@@ -15,19 +15,18 @@
 #include "Passwd.h"
 #include "entropy.h"
 
+#include <openssl/opensslv.h>
 #include <openssl/rand.h>
 #include <openssl/engine.h>
 #include <openssl/evp.h>
-#include <QPushButton>
-#include <QMessageBox>
 #include <QThread>
 
-#include <ltdl.h>
-#include "ui_SelectToken.h"
-#include "widgets/PwDialog.h"
-#include "widgets/XcaWarning.h"
+#include "PwDialogCore.h"
+#include "XcaWarningCore.h"
 
-#include "openssl_compat.h"
+#pragma message ("split PwDialog into console and GUI")
+#include "ui_SelectToken.h"
+#include <QPushButton>
 
 void waitcursor(int start, int line)
 {
@@ -102,7 +101,7 @@ QList<CK_MECHANISM_TYPE> pkcs11::mechanismList(const slotid &slot)
 	CALL_P11_C(slot.lib, C_GetMechanismList, slot.id, NULL, &count);
 	if (count != 0) {
 		m = (CK_MECHANISM_TYPE *)malloc(count *sizeof(*m));
-		check_oom(m);
+		Q_CHECK_PTR(m);
 
 		CALL_P11_C(slot.lib, C_GetMechanismList, slot.id, m, &count);
 		if (rv != CKR_OK) {
@@ -245,7 +244,7 @@ QString pkcs11::tokenLogin(QString name, bool so, bool force)
 			if (!ppt.err.isEmpty())
 				throw errorEx(ppt.err);
 		} else {
-			if (PwDialog::execute(&p, &pin, false) != 1)
+			if (PwDialogCore::execute(&p, &pin, false) != 1)
 				return QString();
 		}
 		login(pin.constUchar(), pin.size(), so);
@@ -337,7 +336,7 @@ void pkcs11::changePin(const slotid &slot, bool so)
 	pass_info p(XCA_TITLE, msg.arg(ti.label()) + "\n" + ti.pinInfo());
 	p.setPin();
 
-	if (PwDialog::execute(&p, &newPin, true) == 1) {
+	if (PwDialogCore::execute(&p, &newPin, true) == 1) {
 		pinp = pin.toLatin1();
 		setPin(pinp.constUchar(), pinp.size(),
 			newPin.constUchar(), newPin.size());
@@ -362,7 +361,7 @@ void pkcs11::initPin(const slotid &slot)
 	p.setPin();
 
 	if (!ti.protAuthPath()) {
-		ret = PwDialog::execute(&p, &newPin, true);
+		ret = PwDialogCore::execute(&p, &newPin, true);
 		pinp = newPin;
 	}
 	p11slot.isValid();
@@ -655,8 +654,28 @@ int pkcs11::encrypt(int flen, const unsigned char *from,
 	return size;
 }
 
-#if OPENSSL_VERSION_NUMBER < 0x10000000L || \
-    OPENSSL_VERSION_NUMBER >= 0x10100000L && ! defined LIBRESSL_VERSION_NUMBER
+// Shared between libressl and openssl
+static int eng_idx = -1;
+static int eng_finish(ENGINE *e)
+{
+	pkcs11 *p11 = (pkcs11 *)ENGINE_get_ex_data(e, eng_idx);
+	delete p11;
+	ENGINE_set_ex_data(e, eng_idx, NULL);
+	return 1;
+}
+
+#if (OPENSSL_VERSION_NUMBER >= 0x30000000L)
+static int eng_pmeth_copy(EVP_PKEY_CTX *dst, const EVP_PKEY_CTX *src)
+#else
+static int eng_pmeth_copy(EVP_PKEY_CTX *dst, EVP_PKEY_CTX *src)
+#endif
+{
+	void *p = EVP_PKEY_CTX_get_app_data((EVP_PKEY_CTX *)src);
+	EVP_PKEY_CTX_set_app_data(dst,  p);
+	return 1;
+}
+
+#ifndef LIBRESSL_VERSION_NUMBER
 static int rsa_privdata_free(RSA *rsa)
 {
 	pkcs11 *priv = (pkcs11*)RSA_get_app_data(rsa);
@@ -720,8 +739,7 @@ out:
 	return NULL;
 }
 
-#if !defined(OPENSSL_NO_EC) && OPENSSL_VERSION_NUMBER >= 0x10100000L
-//	OpenSSL < 1.0.0 have no EC_KEY_METHOD.
+#ifndef OPENSSL_NO_EC
 
 static void ec_privdata_free(EC_KEY *ec)
 {
@@ -817,17 +835,107 @@ static EC_KEY_METHOD *setup_ec_key_meth()
 }
 #endif
 
+#ifdef EVP_PKEY_ED25519
+
+static EVP_PKEY_METHOD *p11_eddsa_method;
+
+static int eddsa_eng_meths(ENGINE *e, EVP_PKEY_METHOD **m, const int **nids, int nid)
+{
+	static const int my_nids[] = {EVP_PKEY_ED25519 };
+	(void)e;
+	if (m) {
+		switch (nid) {
+		case EVP_PKEY_ED25519:
+			*m = p11_eddsa_method;
+			return 1;
+		return 0;
+	    }
+	}
+	if (nids) {
+		*nids = my_nids;
+		return ARRAY_SIZE(my_nids);
+	}
+	return -1;
+
+}
+
+static int eng_pmeth_sign_eddsa(EVP_MD_CTX *ctx,
+			unsigned char *sig, size_t *siglen,
+			const unsigned char *tbs, size_t tbslen)
+{
+	int len, ret = -1;
+	unsigned char rs_buf[64];
+	EVP_PKEY *pkey = EVP_PKEY_CTX_get0_pkey(EVP_MD_CTX_pkey_ctx(ctx));
+	pkcs11 *p11 = (pkcs11 *)ENGINE_get_ex_data(EVP_PKEY_get0_engine(pkey), eng_idx);
+	*siglen = EVP_PKEY_size(pkey);
+	if (sig == NULL) {
+	    // caller needs only size
+	    ret = 1;
+	    goto out;
+	}
+
+	// siglen is unsigned and can' cope with -1 as return value
+	len = p11->encrypt(tbslen, tbs, rs_buf, sizeof rs_buf, CKM_EDDSA);
+	if ((len & 0x01) || (*siglen != len)) // Must be even
+		goto out;
+	memcpy(sig, rs_buf, len);
+	*siglen = len;
+	ret = 1;
+ out:
+	ign_openssl_error();
+	return ret;
+}
+
+static int eng_pmeth_ctrl_eddsa(EVP_PKEY_CTX *ctx, int type, int p1, void *p2)
+{
+	(void)p1;
+	switch (type) {
+	case EVP_PKEY_CTRL_MD:
+		if (p2 == NULL || (const EVP_MD *)p2 == EVP_md_null())
+		    return 1;
+		ECerr(EC_F_PKEY_ECD_CTRL, EC_R_INVALID_DIGEST_TYPE);
+		return 0;
+	}
+	qWarning() << "EC Don't call me" << type;
+	return -2;
+}
+#endif
+
 EVP_PKEY *pkcs11::getPrivateKey(EVP_PKEY *pub, CK_OBJECT_HANDLE obj)
 {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-	static RSA_METHOD rsa_meth_buf;
-	static DSA_METHOD dsa_meth_buf;
-#endif
 	static RSA_METHOD *rsa_meth = NULL;
 	static DSA_METHOD *dsa_meth = NULL;
-#if !defined(OPENSSL_NO_EC) && OPENSSL_VERSION_NUMBER >= 0x10100000L
+#ifndef OPENSSL_NO_EC
 	static EC_KEY_METHOD *ec_key_meth = NULL;
 	EC_KEY *ec;
+#endif
+#ifdef EVP_PKEY_ED25519
+	static ENGINE *e = NULL;
+
+	if (!e) {
+		e = ENGINE_new();
+		Q_CHECK_PTR(e);
+
+		ENGINE_set_pkey_meths(e, eddsa_eng_meths);
+		ENGINE_set_finish_function(e, eng_finish);
+		if (eng_idx == -1)
+			eng_idx = ENGINE_get_ex_new_index(0, NULL, NULL, NULL, 0);
+		ENGINE_set_ex_data(e, eng_idx, NULL);
+		// Why is engine attached to pubkey? I'm commenting it, as I do
+		// not want it to be attached to RSA/DSA/EC
+		//CRYPTO_add(&pub->references, 1, CRYPTO_LOCK_EVP_PKEY);
+		//pub->engine = e;
+
+		if (!p11_eddsa_method) {
+			p11_eddsa_method = EVP_PKEY_meth_new(EVP_PKEY_ED25519,
+					EVP_PKEY_FLAG_SIGCTX_CUSTOM);
+			EVP_PKEY_meth_set_digestsign(p11_eddsa_method,
+					eng_pmeth_sign_eddsa);
+			EVP_PKEY_meth_set_ctrl(p11_eddsa_method,
+					eng_pmeth_ctrl_eddsa, NULL);
+			EVP_PKEY_meth_set_copy(p11_eddsa_method, eng_pmeth_copy);
+		}
+	}
 #endif
 	RSA *rsa;
 	DSA *dsa;
@@ -840,22 +948,13 @@ EVP_PKEY *pkcs11::getPrivateKey(EVP_PKEY *pub, CK_OBJECT_HANDLE obj)
 
 	switch (EVP_PKEY_type(keytype)) {
 	case EVP_PKEY_RSA:
-		rsa = EVP_PKEY_get0_RSA(pub);
-		rsa = RSAPublicKey_dup(rsa);
+		rsa = RSAPublicKey_dup(EVP_PKEY_get0_RSA(pub));
 		openssl_error();
 		if (!rsa_meth) {
-#if OPENSSL_VERSION_NUMBER >= 0x1010000L
 			rsa_meth = RSA_meth_dup(RSA_get_default_method());
 			RSA_meth_set_priv_enc(rsa_meth, rsa_encrypt);
 			RSA_meth_set_priv_dec(rsa_meth, rsa_decrypt);
 			RSA_meth_set_finish(rsa_meth, rsa_privdata_free);
-#else
-			rsa_meth = &rsa_meth_buf;
-			*rsa_meth = *RSA_get_default_method();
-			rsa_meth->rsa_priv_enc = rsa_encrypt;
-			rsa_meth->rsa_priv_dec = rsa_decrypt;
-			rsa_meth->finish = rsa_privdata_free;
-#endif
 		}
 		p11obj = obj;
 		RSA_set_method(rsa, rsa_meth);
@@ -865,20 +964,12 @@ EVP_PKEY *pkcs11::getPrivateKey(EVP_PKEY *pub, CK_OBJECT_HANDLE obj)
 		EVP_PKEY_assign_RSA(evp, rsa);
 		break;
 	case EVP_PKEY_DSA:
-		dsa = EVP_PKEY_get0_DSA(pub);
-		dsa = DSAparams_dup(dsa);
+		dsa = DSAparams_dup(EVP_PKEY_get0_DSA(pub));
 		openssl_error();
 		if (!dsa_meth) {
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
 			dsa_meth = DSA_meth_dup(DSA_get_default_method());
 			DSA_meth_set_sign(dsa_meth, dsa_sign);
 			DSA_meth_set_finish(dsa_meth, dsa_privdata_free);
-#else
-			dsa_meth = &dsa_meth_buf;
-			*dsa_meth = *DSA_get_default_method();
-			dsa_meth->dsa_do_sign = dsa_sign;
-			dsa_meth->finish = dsa_privdata_free;
-#endif
 		}
 		p11obj = obj;
 		DSA_set_method(dsa, dsa_meth);
@@ -887,10 +978,9 @@ EVP_PKEY *pkcs11::getPrivateKey(EVP_PKEY *pub, CK_OBJECT_HANDLE obj)
 		openssl_error();
 		EVP_PKEY_assign_DSA(evp, dsa);
 		break;
-#if !defined(OPENSSL_NO_EC) && OPENSSL_VERSION_NUMBER >= 0x10100000L
+#ifndef OPENSSL_NO_EC
 	case EVP_PKEY_EC:
-		ec = EVP_PKEY_get0_EC_KEY(pub);
-		ec = EC_KEY_dup(ec);
+		ec = EC_KEY_dup(EVP_PKEY_get0_EC_KEY(pub));
 		openssl_error();
 		if (!ec_key_meth) {
 			ec_key_meth = setup_ec_key_meth();
@@ -902,27 +992,35 @@ EVP_PKEY *pkcs11::getPrivateKey(EVP_PKEY *pub, CK_OBJECT_HANDLE obj)
 		openssl_error();
 		EVP_PKEY_assign_EC_KEY(evp, ec);
 		break;
+#endif
+#ifdef EVP_PKEY_ED25519
+	case EVP_PKEY_ED25519:
+		size_t len;
+		if (ENGINE_get_ex_data(e, eng_idx))
+			qWarning() << "We forgot to free the previous Card key.";
+		ENGINE_set_ex_data(e, eng_idx, this);
+		p11obj = obj;
+		EVP_PKEY_get_raw_public_key(pub, NULL, &len);
+		unsigned char *pubkey = (unsigned char *)OPENSSL_malloc(len);
+		Q_CHECK_PTR(pubkey);
+		EVP_PKEY_get_raw_public_key(pub, pubkey, &len);
+		evp = EVP_PKEY_new_raw_public_key(EVP_PKEY_ED25519, e, pubkey, len);
+		openssl_error();
+		OPENSSL_free(pubkey);
+		//EVP_PKEY_set1_engine(evp, e);
+		break;
 #endif
 	}
 	return evp;
 }
 #else
 
-static int eng_idx = -1;
 static EVP_PKEY_METHOD *p11_rsa_method;
 static EVP_PKEY_METHOD *p11_dsa_method;
 #ifndef OPENSSL_NO_EC
 static EVP_PKEY_METHOD *p11_ec_method;
 #endif
 
-static int eng_finish(ENGINE *e)
-{
-	pkcs11 *p11 = (pkcs11 *)ENGINE_get_ex_data(e, eng_idx);
-	delete p11;
-	ENGINE_set_ex_data(e, eng_idx, NULL);
-	return 1;
-}
-
 static int eng_pmeth_ctrl_rsa(EVP_PKEY_CTX *ctx, int type, int p1, void *p2)
 {
 	switch (type) {
@@ -1023,7 +1121,7 @@ static unsigned char *create_x509_sig(EVP_PKEY_CTX *ctx, const unsigned char *m,
 	if (i <= 0)
 		return NULL;
 	p = tmps = (unsigned char *)malloc(i);
-	check_oom(tmps);
+	Q_CHECK_PTR(tmps);
 	if (i2d_X509_SIG(&sig, &p) <= 0) {
 		free(tmps);
 		return NULL;
@@ -1135,13 +1233,6 @@ out:
 }
 #endif
 
-static int eng_pmeth_copy(EVP_PKEY_CTX *dst, EVP_PKEY_CTX *src)
-{
-	void *p = EVP_PKEY_CTX_get_app_data(src);
-	EVP_PKEY_CTX_set_app_data(dst,  p);
-	return 1;
-}
-
 static int eng_meths(ENGINE *e, EVP_PKEY_METHOD **m, const int **nids, int nid)
 {
 	static const int my_nids[] = { EVP_PKEY_EC, EVP_PKEY_RSA };
@@ -1175,7 +1266,7 @@ EVP_PKEY *pkcs11::getPrivateKey(EVP_PKEY *pub, CK_OBJECT_HANDLE obj)
 
 	if (!e) {
 		e = ENGINE_new();
-		check_oom(e);
+		Q_CHECK_PTR(e);
 
 		ENGINE_set_pkey_meths(e, eng_meths);
 		ENGINE_set_finish_function(e, eng_finish);
