@@ -9,12 +9,13 @@
 #include "pki_evp.h"
 #include "pass_info.h"
 #include "func.h"
-#include "db.h"
 #include "entropy.h"
 #include "BioByteArray.h"
-#include "widgets/PwDialog.h"
-#include "widgets/XcaWarning.h"
-#include "widgets/XcaProgress.h"
+#include "XcaProgress.h"
+#include "openssl_compat.h"
+
+#include "PwDialogCore.h"
+#include "XcaWarningCore.h"
 
 #include <openssl/rand.h>
 #include <openssl/evp.h>
@@ -22,10 +23,7 @@
 #include <openssl/pkcs12.h>
 #include <openssl/err.h>
 
-#include "openssl_compat.h"
-
 Passwd pki_evp::passwd;
-Passwd pki_evp::oldpasswd;
 
 QString pki_evp::passHash = QString();
 
@@ -90,11 +88,7 @@ void pki_evp::generate(const keyjob &task)
 	Entropy::seed_rng();
 	XcaProgress progress;
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-	BN_GENCB _bar, *bar = &_bar;
-#else
 	BN_GENCB *bar = BN_GENCB_new();
-#endif
 	BN_GENCB_set_old(bar, XcaProgress::inc, &progress);
 
 	switch (task.ktype.type) {
@@ -145,6 +139,7 @@ void pki_evp::generate(const keyjob &task)
 	case EVP_PKEY_ED25519: {
 		EVP_PKEY *pkey = NULL;
 		EVP_PKEY_CTX *pctx = EVP_PKEY_CTX_new_id(EVP_PKEY_ED25519, NULL);
+		Q_CHECK_PTR(pctx);
 		EVP_PKEY_keygen_init(pctx);
 		EVP_PKEY_keygen(pctx, &pkey);
 		EVP_PKEY_CTX_free(pctx);
@@ -154,9 +149,7 @@ void pki_evp::generate(const keyjob &task)
 #endif
 #endif
 	}
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	BN_GENCB_free(bar);
-#endif
 	isPub = false;
 	pkiSource = generated;
 	pki_openssl_error();
@@ -181,7 +174,6 @@ pki_evp::pki_evp(const QString &n, int type)
 	pki_openssl_error();
 }
 
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
 static bool EVP_PKEY_isPrivKey(EVP_PKEY *key)
 {
 	const BIGNUM *b;
@@ -212,28 +204,6 @@ static bool EVP_PKEY_isPrivKey(EVP_PKEY *key)
 	return false;
 }
 
-#else
-
-static bool EVP_PKEY_isPrivKey(EVP_PKEY *key)
-{
-	int keytype;
-
-	keytype = EVP_PKEY_id(key);
-
-	switch (EVP_PKEY_type(keytype)) {
-		case EVP_PKEY_RSA:
-			return key->pkey.rsa->d ? true: false;
-		case EVP_PKEY_DSA:
-			return key->pkey.dsa->priv_key ? true: false;
-#ifndef OPENSSL_NO_EC
-		case EVP_PKEY_EC:
-			return EC_KEY_get0_private_key(key->pkey.ec) ? true: false;
-#endif
-	}
-	return false;
-}
-#endif
-
 pki_evp::pki_evp(EVP_PKEY *pkey)
 	:pki_key()
 {
@@ -263,7 +233,7 @@ void pki_evp::fromPEMbyteArray(const QByteArray &ba, const QString &name)
 
 	while (!pkey) {
 		pkey = PEM_read_bio_PrivateKey(BioByteArray(ba).ro(), NULL,
-						PwDialog::pwCallback, &p);
+						PwDialogCore::pwCallback, &p);
 		if (openssl_pw_error())
 			XCA_PASSWD_ERROR();
 		if (p.getResult() != pw_ok)
@@ -282,8 +252,8 @@ void pki_evp::fromPEMbyteArray(const QByteArray &ba, const QString &name)
 static void search_ec_oid(EVP_PKEY *pkey)
 {
 #ifndef OPENSSL_NO_EC
-	EC_KEY *ec;
 	EC_GROUP *builtin;
+	const EC_KEY *ec;
 	const EC_GROUP *ec_group;
 
 	int keytype = EVP_PKEY_id(pkey);
@@ -380,7 +350,7 @@ EVP_PKEY *pki_evp::load_ssh_ed25519_privatekey(const QByteArray &ba,
 	pub = ssh_key_next_chunk(&content);
 	ssh_key_check_chunk(&pub, "ssh-ed25519");
 	pub = ssh_key_next_chunk(&pub);
-	if (pub.count() != ED25519_KEYLEN)
+	if (pub.size() != ED25519_KEYLEN)
 		return NULL;
 
 	// Followed by the private key
@@ -395,7 +365,7 @@ EVP_PKEY *pki_evp::load_ssh_ed25519_privatekey(const QByteArray &ba,
 		return NULL;
 	priv = ssh_key_next_chunk(&priv);
 	// The private key is concatenated by the public key in one chunk
-	if (priv.count() != 2 * ED25519_KEYLEN)
+	if (priv.size() != 2 * ED25519_KEYLEN)
 		return NULL;
 	// The last ED25519_KEYLEN bytes must match the public key
 	if (pub != priv.mid(ED25519_KEYLEN))
@@ -415,78 +385,68 @@ void pki_evp::fload(const QString &fname)
 {
 	pass_info p(XCA_TITLE, tr("Please enter the password to decrypt the private key from file:\n%1").
 		arg(compressFilename(fname)));
-	pem_password_cb *cb = PwDialog::pwCallback;
+	pem_password_cb *cb = PwDialogCore::pwCallback;
 
 	pki_ign_openssl_error();
 	XFile file(fname);
 	file.open_read();
+	QByteArray ba = file.readAll();
 	EVP_PKEY *pkey;
 
 	do {
-		pkey = PEM_read_PrivateKey(file.fp(), NULL, cb, &p);
+		pkey = PEM_read_bio_PrivateKey(BioByteArray(ba).ro(),
+						NULL, cb, &p);
 		if (openssl_pw_error())
 			XCA_PASSWD_ERROR();
 		if (p.getResult() != pw_ok)
 			throw p.getResult();
 		if (pki_ign_openssl_error())
 			break;
-		file.retry_read();
 	} while (!pkey);
 
 	if (!pkey) {
 		pki_ign_openssl_error();
-		file.retry_read();
-		pkey = d2i_PrivateKey_fp(file.fp(), NULL);
+		pkey = d2i_PrivateKey_bio(BioByteArray(ba).ro(), NULL);
 	}
 	if (!pkey) {
 		pki_ign_openssl_error();
-		file.retry_read();
-		pkey = d2i_PKCS8PrivateKey_fp(file.fp(), NULL, cb, &p);
+		pkey = d2i_PKCS8PrivateKey_bio(BioByteArray(ba).ro(),
+						NULL, cb, &p);
 	}
 	if (!pkey) {
 		PKCS8_PRIV_KEY_INFO *p8inf;
 		pki_ign_openssl_error();
-		file.retry_read();
-		p8inf = d2i_PKCS8_PRIV_KEY_INFO_fp(file.fp(), NULL);
+		p8inf = d2i_PKCS8_PRIV_KEY_INFO_bio(BioByteArray(ba).ro(),
+							NULL);
 		if (p8inf) {
 			pkey = EVP_PKCS82PKEY(p8inf);
 			PKCS8_PRIV_KEY_INFO_free(p8inf);
 		}
 	}
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
 	if (!pkey) {
 		pki_ign_openssl_error();
-		file.retry_read();
-		pkey = b2i_PVK_bio(file.bio(), cb, &p);
+		pkey = b2i_PVK_bio(BioByteArray(ba).ro(), cb, &p);
 	}
 	if (!pkey) {
 		pki_ign_openssl_error();
-		file.retry_read();
-		pkey = load_ssh_ed25519_privatekey(file.read(10000), p);
-	}
-#endif
-	if (!pkey) {
-		pki_ign_openssl_error();
-		file.retry_read();
-		pkey = PEM_read_PUBKEY(file.fp(), NULL, cb, &p);
+		pkey = load_ssh_ed25519_privatekey(ba, p);
 	}
 	if (!pkey) {
 		pki_ign_openssl_error();
-		file.retry_read();
-		pkey = d2i_PUBKEY_fp(file.fp(), NULL);
+		pkey = PEM_read_bio_PUBKEY(BioByteArray(ba).ro(), NULL, cb, &p);
 	}
 	if (!pkey) {
 		pki_ign_openssl_error();
-		file.retry_read();
-		pkey = load_ssh2_key(file);
+		pkey = d2i_PUBKEY_bio(BioByteArray(ba).ro(), NULL);
 	}
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
 	if (!pkey) {
 		pki_ign_openssl_error();
-		file.retry_read();
-		pkey = b2i_PublicKey_bio(file.bio());
+		pkey = load_ssh2_key(ba);
+	}
+	if (!pkey) {
+		pki_ign_openssl_error();
+		pkey = b2i_PublicKey_bio(BioByteArray(ba).ro());
 	}
-#endif
 	if (pki_ign_openssl_error() || !pkey) {
 		if (pkey)
 			EVP_PKEY_free(pkey);
@@ -495,38 +455,6 @@ void pki_evp::fload(const QString &fname)
 	set_EVP_PKEY(pkey, fname);
 }
 
-void pki_evp::fromData(const unsigned char *p, db_header_t *head)
-{
-	int version, type, size;
-	void *ptr = NULL;
-
-	if (key)
-		EVP_PKEY_free(key);
-	key = NULL;
-
-	size = head->len - sizeof(db_header_t);
-	version = head->version;
-
-	QByteArray ba((const char*)p, size);
-
-	type = db::intFromData(ba);
-	ownPass = (enum passType)db::intFromData(ba);
-	if (version < 2) {
-		d2i_old(ba, type);
-	} else {
-		d2i(ba);
-	}
-	pki_openssl_error();
-
-	if (key)
-		ptr = EVP_PKEY_get0(key);
-	if (!ptr)
-		throw errorEx(tr("Ignoring unsupported private key"));
-
-	encKey = ba;
-	isPub = encKey.size() == 0;
-}
-
 EVP_PKEY *pki_evp::decryptKey() const
 {
 	Passwd ownPassBuf;
@@ -539,7 +467,7 @@ EVP_PKEY *pki_evp::decryptKey() const
 	/* This key has its own password */
 	if (ownPass == ptPrivate) {
 		pass_info pi(XCA_TITLE, tr("Please enter the password to decrypt the private key: '%1'").arg(getIntName()));
-		ret = PwDialog::execute(&pi, &ownPassBuf, false);
+		ret = PwDialogCore::execute(&pi, &ownPassBuf, false);
 		if (ret != 1)
 			throw errorEx(tr("Password input aborted"),
 					getClassName());
@@ -552,7 +480,7 @@ EVP_PKEY *pki_evp::decryptKey() const
 			 sha512passwd(ownPassBuf, passHash) != passHash))
 		{
 			pass_info p(XCA_TITLE, tr("Please enter the database password for decrypting the key '%1'").arg(getIntName()));
-			ret = PwDialog::execute(&p, &ownPassBuf,
+			ret = PwDialogCore::execute(&p, &ownPassBuf,
 							passHash.isEmpty());
 			if (ret != 1)
 				throw errorEx(tr("Password input aborted"),
@@ -560,8 +488,8 @@ EVP_PKEY *pki_evp::decryptKey() const
 		}
 	}
 	QByteArray myencKey = getEncKey();
-	qDebug() << "myencKey.count()"<<myencKey.count();
-	if (myencKey.count() == 0)
+	qDebug() << "myencKey.size()"<<myencKey.size();
+	if (myencKey.size() == 0)
 		return NULL;
 	EVP_PKEY *priv = NULL;
 	X509_SIG *p8 = d2i_PKCS8_bio(BioByteArray(myencKey).ro(), NULL);
@@ -574,58 +502,8 @@ EVP_PKEY *pki_evp::decryptKey() const
 		}
 		X509_SIG_free(p8);
 	}
-	if (priv)
-		return priv;
 	pki_ign_openssl_error();
-	return legacyDecryptKey(myencKey, ownPassBuf);
-}
-
-EVP_PKEY *pki_evp::legacyDecryptKey(QByteArray &myencKey,
-				    Passwd &ownPassBuf) const
-{
-	unsigned char *p;
-	const unsigned char *p1;
-	int outl, decsize;
-	unsigned char iv[EVP_MAX_IV_LENGTH];
-	unsigned char ckey[EVP_MAX_KEY_LENGTH];
-
-	EVP_PKEY *tmpkey;
-	EVP_CIPHER_CTX *ctx;
-	const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
-	p = (unsigned char *)OPENSSL_malloc(myencKey.count());
-	check_oom(p);
-	pki_openssl_error();
-	p1 = p;
-	memset(iv, 0, EVP_MAX_IV_LENGTH);
-
-	memcpy(iv, myencKey.constData(), 8); /* recover the iv */
-	/* generate the key */
-	EVP_BytesToKey(cipher, EVP_sha1(), iv,
-		ownPassBuf.constUchar(), ownPassBuf.size(), 1, ckey, NULL);
-	ctx = EVP_CIPHER_CTX_new();
-	EVP_DecryptInit(ctx, cipher, ckey, iv);
-	EVP_DecryptUpdate(ctx, p , &outl,
-		(const unsigned char*)myencKey.constData() +8,
-		myencKey.count() -8);
-	decsize = outl;
-	EVP_DecryptFinal_ex(ctx, p + decsize , &outl);
-
-	EVP_CIPHER_CTX_cleanup(ctx);
-	decsize += outl;
-	pki_openssl_error();
-	tmpkey = d2i_PrivateKey(getKeyType(), NULL, &p1, decsize);
-	pki_openssl_error();
-	OPENSSL_cleanse(p, myencKey.count());
-	OPENSSL_free(p);
-	EVP_CIPHER_CTX_free(ctx);
-
-	pki_openssl_error();
-	if (EVP_PKEY_type(getKeyType()) == EVP_PKEY_RSA) {
-		RSA *rsa = EVP_PKEY_get0_RSA(tmpkey);
-		RSA_blinding_on(rsa, NULL);
-	}
-	myencKey.fill(0);
-	return tmpkey;
+	return priv;
 }
 
 EVP_PKEY *pki_evp::priv2pub(EVP_PKEY* key)
@@ -636,7 +514,7 @@ EVP_PKEY *pki_evp::priv2pub(EVP_PKEY* key)
 
 	keylen = i2d_PUBKEY(key, NULL);
 	p1 = p = (unsigned char *)OPENSSL_malloc(keylen);
-	check_oom(p);
+	Q_CHECK_PTR(p);
 
 	/* convert rsa/dsa/ec to Pubkey */
 	keylen = i2d_PUBKEY(key, &p);
@@ -658,7 +536,7 @@ void pki_evp::encryptKey(const char *password)
 		int ret;
 		pass_info p(XCA_TITLE, tr("Please enter the password to protect the private key: '%1'").
 			arg(getIntName()));
-		ret = PwDialog::execute(&p, &ownPassBuf, true);
+		ret = PwDialogCore::execute(&p, &ownPassBuf, true);
 		if (ret != 1)
 			throw errorEx("Password input aborted", getClassName());
 		pki_openssl_error();
@@ -679,7 +557,7 @@ void pki_evp::encryptKey(const char *password)
 				(sha512passwT(ownPassBuf, passHash) != passHash &&
 				 sha512passwd(ownPassBuf, passHash) != passHash))
 			{
-				ret = PwDialog::execute(&p, &ownPassBuf,
+				ret = PwDialogCore::execute(&p, &ownPassBuf,
 							passHash.isEmpty());
 				if (ret != 1)
 					throw errorEx("Password input aborted",
@@ -699,7 +577,7 @@ void pki_evp::encryptKey(const char *password)
 	   have the encrypted private in "encKey"
 	 */
 	EVP_PKEY *pkey1 = priv2pub(key);
-	check_oom(pkey1);
+	Q_CHECK_PTR(pkey1);
 	EVP_PKEY_free(key);
 	key = pkey1;
 	pki_openssl_error();
@@ -708,7 +586,7 @@ void pki_evp::encryptKey(const char *password)
 void pki_evp::set_evp_key(EVP_PKEY *pkey)
 {
 	if (key)
-		free(key);
+		EVP_PKEY_free(key);
 	key = pkey;
 }
 
@@ -758,7 +636,7 @@ QByteArray pki_evp::getEncKey() const
 	QSqlError e;
 	QByteArray ba;
 
-	if (encKey.count() > 0 || !sqlItemId.isValid())
+	if (encKey.size() > 0 || !sqlItemId.isValid())
 		return encKey;
 
 	SQL_PREPARE(q, "SELECT private FROM private_keys WHERE item=?");
@@ -782,6 +660,55 @@ QSqlError pki_evp::deleteSqlData()
 	return q.lastError();
 }
 
+bool pki_evp::pem(BioByteArray &b)
+{
+	EVP_PKEY *pkey;
+	int keytype;
+	const pki_export *xport = pki_export::by_id(Settings["KeyFormat"]);
+
+	if (xport->match_all(F_PEM | F_PRIVATE)) {
+		pkey = decryptKey();
+		keytype = EVP_PKEY_id(pkey);
+		switch (keytype) {
+		case EVP_PKEY_RSA:
+			PEM_write_bio_RSAPrivateKey(b,
+				EVP_PKEY_get0_RSA(pkey),
+				NULL, NULL, 0, NULL, NULL);
+			break;
+		case EVP_PKEY_DSA:
+			PEM_write_bio_DSAPrivateKey(b,
+				EVP_PKEY_get0_DSA(pkey),
+				NULL, NULL, 0, NULL, NULL);
+			break;
+#ifndef OPENSSL_NO_EC
+		case EVP_PKEY_EC:
+			PEM_write_bio_ECPrivateKey(b,
+				EVP_PKEY_get0_EC_KEY(pkey),
+				NULL, NULL, 0, NULL, NULL);
+			break;
+#ifdef EVP_PKEY_ED25519
+		case EVP_PKEY_ED25519:
+			 if (xport->match_all(F_PRIVATE))
+				return false;
+			write_SSH2_ed25519_private(b, pkey, NULL);
+			break;
+#endif
+#endif
+		}
+		EVP_PKEY_free(pkey);
+	} else if (xport->match_all(F_PKCS8 | F_PRIVATE)) {
+		const EVP_CIPHER *algo = xport->match_all(F_CRYPT) ?
+			EVP_aes_256_cbc() : NULL;
+		pkey = decryptKey();
+        PEM_write_bio_PrivateKey(b, pkey, algo,
+					 passwd.constUchar(), passwd.size(),
+					 NULL, NULL);
+		EVP_PKEY_free(pkey);
+	} else
+		return pki_key::pem(b);
+
+	return true;
+}
 void pki_evp::writePKCS8(XFile &file, const EVP_CIPHER *enc,
 		pem_password_cb *cb, bool pem) const
 {
@@ -793,18 +720,19 @@ void pki_evp::writePKCS8(XFile &file, const EVP_CIPHER *enc,
 		pki_openssl_error();
 		return;
 	}
+	BioByteArray b;
 	if (pem) {
-		PEM_file_comment(file);
-		PEM_write_PKCS8PrivateKey(file.fp(), pkey, enc, NULL, 0,cb,&p);
+		b += PEM_comment();
+		PEM_write_bio_PKCS8PrivateKey(b, pkey, enc, NULL, 0, cb, &p);
 	} else {
-		i2d_PKCS8PrivateKey_fp(file.fp(), pkey, enc, NULL, 0, cb, &p);
+		i2d_PKCS8PrivateKey_bio(b, pkey, enc, NULL, 0, cb, &p);
 	}
 	EVP_PKEY_free(pkey);
+	file.write(b);
 }
 
 void pki_evp::writePVKprivate(XFile &file, pem_password_cb *cb) const
 {
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
 	pass_info p(XCA_TITLE, tr("Please enter the password protecting the Microsoft PVK key '%1'").arg(getIntName()));
 
 	int enc = cb ? 2 /* pvk-strong */ : 0 /* pvk-none */;
@@ -816,19 +744,17 @@ void pki_evp::writePVKprivate(XFile &file, pem_password_cb *cb) const
 	/* In case of success! the error
 	 *   PEMerr(PEM_F_I2B_PVK_BIO, PEM_R_BIO_WRITE_FAILURE)
 	 * is set. Workaround this behavior */
-	if (i2b_PVK_bio(file.bio(), pkey, enc, cb, &p) == -1) {
+	BioByteArray b;
+	if (i2b_PVK_bio(b, pkey, enc, cb, &p) == -1) {
 		pki_openssl_error();
 		PEMerr(PEM_F_I2B_PVK_BIO, PEM_R_BIO_WRITE_FAILURE);
 		pki_openssl_error();
 	}
 	ign_openssl_error();
 	EVP_PKEY_free(pkey);
-#else
-	(void)file;
-	(void)cb;
-	throw errorEx("Internal Error");
-#endif
+	file.write(b);
 }
+
 static int mycb(char *buf, int size, int, void *)
 {
 	strncpy(buf, pki_evp::passwd, size);
@@ -839,11 +765,11 @@ void pki_evp::writeDefault(const QString &dirname) const
 {
 	XFile file(get_dump_filename(dirname, ".pem"));
 	file.open_key();
-	writeKey(file, pki_evp::passwd[0] ? EVP_des_ede3_cbc() : NULL,
+	writeKey(file, pki_evp::passwd.isEmpty() ? NULL : EVP_des_ede3_cbc(),
 			mycb, true);
 }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined LIBRESSL_VERSION_NUMBER
+#ifndef LIBRESSL_VERSION_NUMBER
 int PEM_write_bio_PrivateKey_traditional(BIO *bp, EVP_PKEY *x,
                                          const EVP_CIPHER *enc,
                                          unsigned char *kstr, int klen,
@@ -872,67 +798,31 @@ void pki_evp::writeKey(XFile &file, const EVP_CIPHER *enc,
 	        pki_openssl_error();
 		return;
 	}
+	BioByteArray b;
 	if (pem) {
-		PEM_file_comment(file);
-		PEM_write_bio_PrivateKey_traditional(file.bio(), pkey, enc,
+		b += PEM_comment();
+		PEM_write_bio_PrivateKey_traditional(b, pkey, enc,
 						NULL, 0, cb, &p);
 	} else {
-		i2d_PrivateKey_fp(file.fp(), pkey);
+		i2d_PrivateKey_bio(b, pkey);
 	}
 	EVP_PKEY_free(pkey);
 	pki_openssl_error();
+	file.write(b);
 }
 
-bool pki_evp::verify_priv(EVP_PKEY *pkey) const
+bool pki_evp::verify(EVP_PKEY *pkey) const
 {
-	bool verify = true;
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
-	unsigned char data[32], sig[1024];
-	size_t datalen = sizeof data, siglen = sizeof sig;
-	EVP_MD_CTX *ctx = NULL;
-	const EVP_MD *md = EVP_sha256();
-	EVP_PKEY_CTX *pkctx = NULL;
-
 	if (!EVP_PKEY_isPrivKey(pkey))
-		return true;
-	do {
-		ctx = EVP_MD_CTX_new();
+		return pki_key::verify(pkey);
+
+	EVP_PKEY_CTX *ctx = EVP_PKEY_CTX_new(pkey, NULL);
+	Q_CHECK_PTR(ctx);
+	int verify = EVP_PKEY_check(ctx);
+	EVP_PKEY_CTX_free(ctx);
+	if (verify == -2) {
+		// Operation not supported assume true
 		pki_ign_openssl_error();
-		RAND_bytes(data, datalen);
-		check_oom(ctx);
-		verify = false;
-
-		/* Sign some random data in "data" */
-#ifdef EVP_PKEY_ED25519
-		if (EVP_PKEY_id(pkey) == EVP_PKEY_ED25519)
-			md = NULL;
-#endif
-		if (!EVP_DigestSignInit(ctx, &pkctx, md, NULL, pkey))
-			break;
-
-		if (EVP_PKEY_id(pkey) == EVP_PKEY_RSA)
-			EVP_PKEY_CTX_set_rsa_padding(pkctx, RSA_PKCS1_PADDING);
-
-		if (!EVP_DigestSign(ctx, sig, &siglen, data, datalen))
-			break;
-
-		/* Verify the signature */
-		if (!EVP_DigestVerifyInit(ctx, NULL, md, NULL, pkey))
-			break;
-
-		if (EVP_DigestVerify(ctx, sig, siglen, data, datalen) != 1)
-			break;
-
-		verify = true;
-	} while (0);
-
-	if (ctx)
-		EVP_MD_CTX_free(ctx);
-#endif
-	if (EVP_PKEY_id(pkey) == EVP_PKEY_RSA && EVP_PKEY_isPrivKey(pkey)) {
-		RSA *rsa = EVP_PKEY_get0_RSA(pkey);
-		if (RSA_check_key(rsa) != 1)
-			verify = false;
 	}
 	pki_openssl_error();
 	return verify;
