@@ -9,17 +9,15 @@
 #include "pki_key.h"
 #include "pki_x509super.h"
 #include "func.h"
-#include "db.h"
 #include "pkcs11.h"
-#include "widgets/XcaWarning.h"
-#include "widgets/ExportDialog.h"
+#include "pki_export.h"
+#include "XcaWarningCore.h"
 
 #include <openssl/rand.h>
 #include <openssl/pem.h>
 
-#include "openssl_compat.h"
-
 builtin_curves builtinCurves;
+keyjob keyjob::defaultjob;
 
 pki_key::pki_key(const QString &name)
         :pki_base(name)
@@ -75,7 +73,7 @@ void pki_key::d2i_old(QByteArray &ba, int type)
 {
 	const unsigned char *p, *p1;
 	p = p1 = (const unsigned char *)ba.constData();
-	EVP_PKEY *k = d2i_PublicKey(type, NULL, &p1, ba.count());
+	EVP_PKEY *k = d2i_PublicKey(type, NULL, &p1, ba.size());
 
         pki_openssl_error();
 
@@ -95,11 +93,11 @@ QByteArray pki_key::i2d() const
 void pki_key::write_SSH2_ed25519_private(BIO *b,
 			 const EVP_PKEY *pkey, const EVP_CIPHER *enc) const
 {
+	(void)enc;
 #ifndef OPENSSL_NO_EC
 	static const char data0001[] = { 0, 0, 0, 1};
 	char buf_nonce[8];
 	QByteArray data, priv, pubfull;
-	(void)enc;
 
 	pubfull = SSH2publicQByteArray(true);
 	RAND_bytes((unsigned char*)buf_nonce, sizeof buf_nonce);
@@ -119,62 +117,23 @@ void pki_key::write_SSH2_ed25519_private(BIO *b,
 	PEM_write_bio(b, PEM_STRING_OPENSSH_KEY, (char*)"",
 		(unsigned char*)(data.data()), data.size());
 	pki_openssl_error();
+#else
+	(void)b;
+	(void)pkey;
 #endif
 }
 
-bool pki_key::pem(BioByteArray &b, int format)
+bool pki_key::pem(BioByteArray &b)
 {
-	EVP_PKEY *pkey;
-	QByteArray ba;
-	int keytype;
+	const pki_export *xport = pki_export::by_id(Settings["KeyFormat"]);
 
-	switch (format) {
-	case exportType::SSH2_public:
-		b += SSH2publicQByteArray();
-		break;
-	case exportType::PEM_private:
-	case exportType::SSH2_private:
-		pkey = decryptKey();
-		keytype = EVP_PKEY_id(pkey);
-		switch (keytype) {
-		case EVP_PKEY_RSA:
-			PEM_write_bio_RSAPrivateKey(b,
-				EVP_PKEY_get0_RSA(pkey),
-				NULL, NULL, 0, NULL, NULL);
-			break;
-		case EVP_PKEY_DSA:
-			PEM_write_bio_DSAPrivateKey(b,
-				EVP_PKEY_get0_DSA(pkey),
-				NULL, NULL, 0, NULL, NULL);
-			break;
-#ifndef OPENSSL_NO_EC
-		case EVP_PKEY_EC:
-			PEM_write_bio_ECPrivateKey(b,
-				EVP_PKEY_get0_EC_KEY(pkey),
-				NULL, NULL, 0, NULL, NULL);
-			break;
-#ifdef EVP_PKEY_ED25519
-		case EVP_PKEY_ED25519:
-			if (format == exportType::PEM_private)
-				return false;
-			write_SSH2_ed25519_private(b, pkey, NULL);
-			break;
-#endif
-#endif
-		}
-		EVP_PKEY_free(pkey);
-		break;
-	case exportType::PKCS8:
-		pkey = decryptKey();
-		PEM_write_bio_PrivateKey(b, pkey, NULL, NULL, 0, NULL, NULL);
-		EVP_PKEY_free(pkey);
-		break;
-	case exportType::PEM_key:
-		PEM_write_bio_PUBKEY(b, key);
-		break;
-	default:
+	if (xport->match_all(F_PRIVATE))
 		return false;
-	}
+	if (xport->match_all(F_SSH2))
+		b += SSH2publicQByteArray();
+	else if (xport->match_all(F_PEM))
+		PEM_write_bio_PUBKEY(b, key);
+
 	return true;
 }
 
@@ -188,7 +147,9 @@ void pki_key::writeSSH2private(XFile &file, pem_password_cb *cb) const
 		pki_openssl_error();
 		return;
 	}
-	write_SSH2_ed25519_private(file.bio(), pkey, NULL);
+	BioByteArray b;
+	write_SSH2_ed25519_private(b, pkey, NULL);
+	file.write(b);
 }
 
 QString pki_key::length() const
@@ -197,7 +158,7 @@ QString pki_key::length() const
 
 	if (EVP_PKEY_id(key) == EVP_PKEY_DSA) {
 		const BIGNUM *p = NULL;
-		DSA *dsa = EVP_PKEY_get0_DSA(key);
+		const DSA *dsa = EVP_PKEY_get0_DSA(key);
 		if (dsa)
 			DSA_get0_pqg(dsa, &p, NULL, NULL);
 		dsa_unset = p == NULL;
@@ -299,7 +260,7 @@ QString pki_key::modulus() const
 	if (getKeyType() == EVP_PKEY_RSA) {
 		const BIGNUM *n = NULL;
 
-		RSA *rsa = EVP_PKEY_get0_RSA(key);
+		const RSA *rsa = EVP_PKEY_get0_RSA(key);
 		RSA_get0_key(rsa, &n, NULL, NULL);
 		return BN2QString(n);
 	}
@@ -310,7 +271,7 @@ QString pki_key::pubEx() const
 {
 	if (getKeyType() == EVP_PKEY_RSA) {
 		const BIGNUM *e = NULL;
-		RSA *rsa = EVP_PKEY_get0_RSA(key);
+		const RSA *rsa = EVP_PKEY_get0_RSA(key);
 		RSA_get0_key(rsa, NULL, &e, NULL);
 		return BN2QString(e);
 	}
@@ -321,7 +282,7 @@ QString pki_key::subprime() const
 {
 	if (getKeyType() == EVP_PKEY_DSA) {
 		const BIGNUM *q = NULL;
-		DSA *dsa = EVP_PKEY_get0_DSA(key);
+		const DSA *dsa = EVP_PKEY_get0_DSA(key);
 		if (dsa)
 			DSA_get0_pqg(dsa, NULL, &q, NULL);
 		return BN2QString(q);
@@ -333,7 +294,7 @@ QString pki_key::pubkey() const
 {
 	if (getKeyType() == EVP_PKEY_DSA) {
 		const BIGNUM *pubkey = NULL;
-		DSA *dsa = EVP_PKEY_get0_DSA(key);
+		const DSA *dsa = EVP_PKEY_get0_DSA(key);
 		if (dsa)
 			DSA_get0_key(dsa, &pubkey, NULL);
 		return BN2QString(pubkey);
@@ -421,16 +382,11 @@ QList<int> pki_key::possibleHashNids()
 				NID_sha384 << NID_sha512 << NID_ripemd160;
 			break;
 		case EVP_PKEY_DSA:
-			nids << NID_sha1;
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
-			nids << NID_sha256;
-#endif
+			nids << NID_sha1 << NID_sha256;
 			break;
 		case EVP_PKEY_EC:
-			nids << NID_sha1;
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
-			nids << NID_sha224 << NID_sha256 << NID_sha384 << NID_sha512;
-#endif
+			nids << NID_sha1  << NID_sha224 << NID_sha256 <<
+				NID_sha384 << NID_sha512;
 			break;
 	}
 	return nids;
@@ -452,13 +408,15 @@ bool pki_key::compare(const pki_base *ref) const
 
 void pki_key::writePublic(XFile &file, bool pem) const
 {
+	BioByteArray b;
 	if (pem) {
-		PEM_file_comment(file);
-		PEM_write_PUBKEY(file.fp(), key);
+		b += PEM_comment();
+		PEM_write_bio_PUBKEY(b, key);
 	} else {
-		i2d_PUBKEY_fp(file.fp(), key);
+		i2d_PUBKEY_bio(b, key);
 	}
 	pki_openssl_error();
+	file.write(b);
 }
 
 QString pki_key::BNoneLine(BIGNUM *bn) const
@@ -481,10 +439,10 @@ QString pki_key::BN2QString(const BIGNUM *bn) const
 	char zs[10];
 	int j, size = BN_num_bytes(bn);
 	unsigned char *buf = (unsigned char *)OPENSSL_malloc(size);
-	check_oom(buf);
+	Q_CHECK_PTR(buf);
 	BN_bn2bin(bn, buf);
 	for (j = 0; j< size; j++) {
-		sprintf(zs, "%02X%c",buf[j], ((j+1)%16 == 0) ? '\n' :
+		snprintf(zs, sizeof zs, "%02X%c",buf[j], ((j+1)%16 == 0) ? '\n' :
 				j<size-1 ? ':' : ' ');
 		x += zs;
 	}
@@ -612,7 +570,7 @@ BIGNUM *pki_key::ssh_key_data2bn(QByteArray *ba) const
 	QByteArray chunk = ssh_key_next_chunk(ba);
 	BIGNUM *bn = BN_bin2bn((const unsigned char *)chunk.constData(),
 				chunk.size(), NULL);
-	check_oom(bn);
+	Q_CHECK_PTR(bn);
 	return bn;
 }
 
@@ -635,14 +593,13 @@ QByteArray pki_key::ssh_key_next_chunk(QByteArray *ba) const
 	return chunk;
 }
 
-EVP_PKEY *pki_key::load_ssh2_key(XFile &file)
+EVP_PKEY *pki_key::load_ssh2_key(const QByteArray &b)
 {
 	/* See RFC 4253 Section 6.6 */
-	QByteArray ba;
 	QStringList sl;
 	EVP_PKEY *pk = NULL;
+	QByteArray ba(b);
 
-	ba = file.read(4096);
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
 	sl = QString(ba).split(" ", Qt::SkipEmptyParts);
 #else
@@ -659,10 +616,10 @@ EVP_PKEY *pki_key::load_ssh2_key(XFile &file)
 		BIGNUM *n = ssh_key_data2bn(&ba);
 
 		RSA *rsa = RSA_new();
-		check_oom(rsa);
+		Q_CHECK_PTR(rsa);
 		RSA_set0_key(rsa, n, e, NULL);
 		pk = EVP_PKEY_new();
-		check_oom(pk);
+		Q_CHECK_PTR(pk);
 		EVP_PKEY_assign_RSA(pk, rsa);
 	} else if (sl[0].startsWith("ssh-dss")) {
 		ssh_key_check_chunk(&ba, "ssh-dss");
@@ -671,13 +628,13 @@ EVP_PKEY *pki_key::load_ssh2_key(XFile &file)
 		BIGNUM *g = ssh_key_data2bn(&ba);
 		BIGNUM *pubkey = ssh_key_data2bn(&ba);
 		DSA *dsa = DSA_new();
-		check_oom(dsa);
+		Q_CHECK_PTR(dsa);
 
 		DSA_set0_pqg(dsa, p, q, g);
 		DSA_set0_key(dsa, pubkey, NULL);
 
 		pk = EVP_PKEY_new();
-		check_oom(pk);
+		Q_CHECK_PTR(pk);
 		EVP_PKEY_assign_DSA(pk, dsa);
 #ifndef OPENSSL_NO_EC
 	} else if (sl[0].startsWith("ecdsa-sha2-nistp256")) {
@@ -689,7 +646,7 @@ EVP_PKEY *pki_key::load_ssh2_key(XFile &file)
 		BIGNUM *bn = ssh_key_data2bn(&ba);
 
 		ec = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
-		check_oom(ec);
+		Q_CHECK_PTR(ec);
 		EC_KEY_set_asn1_flag(ec, OPENSSL_EC_NAMED_CURVE);
 		EC_KEY_set_public_key(ec, EC_POINT_bn2point(
 					EC_KEY_get0_group(ec), bn, NULL, NULL));
@@ -697,14 +654,14 @@ EVP_PKEY *pki_key::load_ssh2_key(XFile &file)
 		pki_openssl_error();
 
 		pk = EVP_PKEY_new();
-		check_oom(pk);
+		Q_CHECK_PTR(pk);
 		EVP_PKEY_assign_EC_KEY(pk, ec);
 #ifdef EVP_PKEY_ED25519
 	} else if (sl[0].startsWith("ssh-ed25519")) {
 		ssh_key_check_chunk(&ba, "ssh-ed25519");
 		QByteArray pub = ssh_key_next_chunk(&ba);
 		pk = EVP_PKEY_new_raw_public_key(EVP_PKEY_ED25519, NULL,
-			(const unsigned char *)pub.constData(), pub.count());
+			(const unsigned char *)pub.constData(), pub.size());
 		pki_openssl_error();
 #endif
 #endif
@@ -766,7 +723,7 @@ QByteArray pki_key::SSH2publicQByteArray(bool raw) const
 		txt = "ssh-rsa";
 		ssh_key_QBA2data(txt, &data);
 		{
-			RSA *rsa = EVP_PKEY_get0_RSA(key);
+			const RSA *rsa = EVP_PKEY_get0_RSA(key);
 			const BIGNUM *n, *e;
 			RSA_get0_key(rsa, &n, &e, NULL);
 			ssh_key_bn2data(e, &data);
@@ -777,7 +734,7 @@ QByteArray pki_key::SSH2publicQByteArray(bool raw) const
 		txt = "ssh-dss";
 		ssh_key_QBA2data(txt, &data);
 		{
-			DSA *dsa = EVP_PKEY_get0_DSA(key);
+			const DSA *dsa = EVP_PKEY_get0_DSA(key);
 			const BIGNUM *p, *q, *g, *pubkey;
 			DSA_get0_pqg(dsa, &p, &q, &g);
 			DSA_get0_key(dsa, &pubkey, NULL);
@@ -832,45 +789,16 @@ void pki_key::writeSSH2public(XFile &file) const
 
 bool pki_key::verify(EVP_PKEY *pkey) const
 {
-	bool verify = true;
-	const BIGNUM *a = NULL;
-	const BIGNUM *b = NULL;
-	const BIGNUM *c = NULL;
-
-	switch (EVP_PKEY_type(EVP_PKEY_id(pkey))) {
-	case EVP_PKEY_RSA:
-		RSA_get0_key(EVP_PKEY_get0_RSA(pkey), &a, &b, NULL);
-		verify = a && b;
-		break;
-	case EVP_PKEY_DSA:
-		DSA_get0_pqg(EVP_PKEY_get0_DSA(pkey), &a, &b, &c);
-		verify = a && b && c;
-		break;
-#ifndef OPENSSL_NO_EC
-	case EVP_PKEY_EC:
-		verify = EC_KEY_check_key(EVP_PKEY_get0_EC_KEY(pkey)) == 1;
-		break;
-#ifdef EVP_PKEY_ED25519
-	case EVP_PKEY_ED25519: {
-		size_t len;
-		verify = EVP_PKEY_get_raw_private_key(pkey, NULL, &len) == 1 &&
-				len == ED25519_KEYLEN;
-		break;
+	EVP_PKEY_CTX *ctx = EVP_PKEY_CTX_new(pkey, NULL);
+	Q_CHECK_PTR(ctx);
+	int verify = EVP_PKEY_public_check(ctx);
+	EVP_PKEY_CTX_free(ctx);
+	if (verify == -2) {
+		// Operation not supported assume true
+		pki_ign_openssl_error();
 	}
-#endif
-#endif
-	default:
-		verify = false;
-	}
-	if (verify)
-		verify = verify_priv(pkey);
 	pki_openssl_error();
-	return verify;
-}
-
-bool pki_key::verify_priv(EVP_PKEY *) const
-{
-	return true;
+	return verify != 0;
 }
 
 QString pki_key::fingerprint(const QString &format) const
@@ -909,25 +837,19 @@ QByteArray pki_key::X509_PUBKEY_public_key() const
 	int len;
 
 	X509_PUBKEY_set(&pk, key);
-#if OPENSSL_VERSION_NUMBER < 0x10000000L
-	p = pk->public_key->data;
-	len = pk->public_key->length;
-#else
 	X509_PUBKEY_get0_param(NULL, &p, &len, NULL, pk);
-#endif
 
 	QByteArray data((const char*)p, len);
 	X509_PUBKEY_free(pk);
 	return data;
 }
 
-void pki_key::PEM_file_comment(XFile &file) const
+QByteArray pki_key::PEM_comment() const
 {
 	if (!pem_comment)
-		return;
-	pki_base::PEM_file_comment(file);
-	file.write(QString("%1 %2\n").arg(length(), getTypeString())
-			.toUtf8());
+		return QByteArray();
+	return pki_base::PEM_comment() +
+		QString("%1 %2\n").arg(length(), getTypeString()).toUtf8();
 }
 
 void pki_key::collect_properties(QMap<QString, QString> &prp) const
@@ -949,11 +871,7 @@ void pki_key::print(BioByteArray &bba, enum print_opt opt) const
 	pki_base::print(bba, opt);
 	switch (opt) {
 	case print_openssl_txt:
-#if OPENSSL_VERSION_NUMBER < 0x10000000L
-		bba += "Not supported\n";
-#else
 		EVP_PKEY_print_public(bba, key, 0, NULL);
-#endif
 		break;
 	case print_pem:
 		PEM_write_bio_PUBKEY(bba, key);
