@@ -13,10 +13,9 @@
 #include <openssl/pem.h>
 #include <openssl/evp.h>
 #include "pki_base.h"
+//#include "pki_export.h"
 #include "builtin_curves.h"
 
-#include "openssl_compat.h"
-
 #define PEM_STRING_OPENSSH_KEY "OPENSSH PRIVATE KEY"
 #define DEFAULT_KEY_LENGTH 2048
 #define ED25519_KEYLEN 32
@@ -41,7 +40,7 @@ class keytype
 			keytype(EVP_PKEY_EC, "EC", CKM_EC_KEY_PAIR_GEN,
 				true, false),
 #ifdef EVP_PKEY_ED25519
-			keytype(EVP_PKEY_ED25519, "ED25519", 0, false, false),
+			keytype(EVP_PKEY_ED25519, "ED25519", CKM_VENDOR_DEFINED, false, false),
 #endif
 #endif
 		};
@@ -96,6 +95,7 @@ class keytype
 class keyjob
 {
     public:
+	static keyjob defaultjob;
 	keytype ktype;
 	int size;
 	int ec_nid;
@@ -179,7 +179,7 @@ class pki_key: public pki_base
 		QString BNoneLine(BIGNUM *bn) const;
 		QByteArray SSH2publicQByteArray(bool raw=false) const;
 		QByteArray X509_PUBKEY_public_key() const;
-		void PEM_file_comment(XFile &file) const;
+		QByteArray PEM_comment() const;
 		void collect_properties(QMap<QString, QString> &prp) const;
 
 		BIGNUM *ssh_key_data2bn(QByteArray *ba) const;
@@ -210,8 +210,7 @@ class pki_key: public pki_base
 		bool compare(const pki_base *ref) const;
 		int getKeyType() const;
 		bool isPrivKey() const;
-		bool verify(EVP_PKEY *pkey) const;
-		virtual bool verify_priv(EVP_PKEY *pkey) const;
+		virtual bool verify(EVP_PKEY *pkey) const;
 		int getUcount() const;
 		void setUcount(int c)
 		{
@@ -233,7 +232,7 @@ class pki_key: public pki_base
 		{
 			qFatal("generate in pki_key");
 		}
-		bool pem(BioByteArray &, int);
+		bool pem(BioByteArray &);
 		QVariant column_data(const dbheader *hd) const;
 		QString modulus() const;
 		QString pubEx() const;
@@ -250,7 +249,7 @@ class pki_key: public pki_base
 		void d2i(QByteArray &ba);
 		void d2i_old(QByteArray &ba, int type);
 		QByteArray i2d() const;
-		EVP_PKEY *load_ssh2_key(XFile &file);
+		EVP_PKEY *load_ssh2_key(const QByteArray &ba);
 		void writeSSH2public(XFile &file) const;
 		void writeSSH2private(XFile &file, pem_password_cb *cb) const;
 		QString fingerprint(const QString &format) const;
