@@ -19,7 +19,7 @@
 #include "exception.h"
 #include "func.h"
 #include "xfile.h"
-#include "widgets/XcaWarning.h"
+#include "XcaWarningCore.h"
 #include <QList>
 
 pki_multi::pki_multi(const QString &name)
@@ -41,7 +41,7 @@ void pki_multi::append_item(pki_base *pki)
 {
 	pki_multi *m = dynamic_cast<pki_multi*>(pki);
 	if (m)
-		multi += m;
+		multi += m->multi;
 	else
 		multi << pki;
 }
@@ -156,7 +156,7 @@ void pki_multi::probeAnything(const QString &fname)
 				append_item(item);
 				break;
 			}
-		} catch (errorEx &err) {
+		} catch (errorEx &) {
 			continue;
 		} catch (enum open_result r) {
 			if (r == pw_cancel)
@@ -171,10 +171,15 @@ void pki_multi::probeAnything(const QString &fname)
 
 void pki_multi::print(BioByteArray &bba, enum print_opt opt) const
 {
+	pki_base::print(bba, opt);
 	foreach(pki_base *pki, multi)
 		pki->print(bba, opt);
 }
 
+void pki_multi::collect_properties(QMap<QString, QString> &prp) const
+{
+}
+
 QList<pki_base *> pki_multi::pull()
 {
 	QList<pki_base*> temp = multi;
