@@ -13,16 +13,14 @@
 #include "pass_info.h"
 #include "exception.h"
 #include "func.h"
-#include "widgets/PwDialog.h"
-#include "widgets/XcaWarning.h"
+#include "PwDialogCore.h"
+#include "XcaWarningCore.h"
+
 #include <openssl/err.h>
 #include <openssl/pkcs12.h>
+#include <openssl/x509.h>
 #include <openssl/stack.h>
 
-#include <QMessageBox>
-
-#include "openssl_compat.h"
-
 pki_pkcs12::pki_pkcs12(const QString &d, pki_x509 *acert, pki_key *akey)
 	:pki_multi(d), cert(acert), key(akey)
 {
@@ -38,21 +36,34 @@ pki_pkcs12::pki_pkcs12(const QString &fname)
 	X509 *mycert = NULL;
 	key = NULL; cert = NULL;
 	pass_info p(XCA_TITLE, tr("Please enter the password to decrypt the PKCS#12 file:\n%1").arg(compressFilename(fname)));
+	const X509_ALGOR *macalgid = NULL;
+	const ASN1_INTEGER *maciter = NULL;
+	const ASN1_OBJECT *macobj = NULL;
 
 	setFilename(fname);
 	XFile file(fname);
 	file.open_read();
-	PKCS12 *pkcs12 = d2i_PKCS12_fp(file.fp(), NULL);
+	BioByteArray b(file.readAll());
+
+	PKCS12 *pkcs12 = d2i_PKCS12_bio(b.ro(), NULL);
 	if (pki_ign_openssl_error()) {
 		if (pkcs12)
 			PKCS12_free(pkcs12);
 		throw errorEx(tr("Unable to load the PKCS#12 (pfx) file %1.")
 				.arg(fname));
 	}
+	PKCS12_get0_mac(NULL, &macalgid, NULL, &maciter, pkcs12);
+	if (macalgid)
+		X509_ALGOR_get0(&macobj, NULL, NULL, macalgid);
+	if (macobj) {
+		algorithm = OBJ_obj2QString(macobj);
+		if (maciter)
+			algorithm += QString(", iteration %1").arg(a1int(maciter).toDec());
+	}
 	while (!PKCS12_verify_mac(pkcs12, pass.constData(), pass.size())) {
 		if (pass.size() > 0)
 			XCA_PASSWD_ERROR();
-		enum open_result result = PwDialog::execute(&p, &pass);
+		enum open_result result = PwDialogCore::execute(&p, &pass);
 		if (result != pw_ok) {
 			/* cancel pressed */
 			PKCS12_free(pkcs12);
@@ -80,7 +91,7 @@ pki_pkcs12::pki_pkcs12(const QString &fname)
 			alias = QString::fromUtf8((const char *)str);
 		alias = QString::fromUtf8(alias.toLatin1());
 		cert = new pki_x509(mycert);
-		check_oom(cert);
+		Q_CHECK_PTR(cert);
 		if (alias.isEmpty()) {
 			cert->autoIntName(fname);
 			alias = cert->getIntName();
@@ -93,7 +104,7 @@ pki_pkcs12::pki_pkcs12(const QString &fname)
 	}
 	if (mykey) {
 		key = new pki_evp(mykey);
-		check_oom(key);
+		Q_CHECK_PTR(key);
 		key->setIntName(alias + "_key");
 		key->pkiSource = imported;
 		inheritFilename(key);
@@ -104,7 +115,7 @@ pki_pkcs12::pki_pkcs12(const QString &fname)
 		if (!crt)
 			continue;
 		pki_x509 *cacert = new pki_x509(crt);
-		check_oom(cacert);
+		Q_CHECK_PTR(cacert);
 		if (alias.isEmpty()) {
 			cacert->autoIntName(QString());
 		} else {
@@ -119,7 +130,7 @@ pki_pkcs12::pki_pkcs12(const QString &fname)
 	pki_openssl_error();
 }
 
-void pki_pkcs12::writePKCS12(XFile &file) const
+void pki_pkcs12::writePKCS12(XFile &file, encAlgo &encAlgo) const
 {
 	Passwd pass;
 	PKCS12 *pkcs12;
@@ -129,7 +140,7 @@ void pki_pkcs12::writePKCS12(XFile &file) const
 	if (cert == NULL || key == NULL)
 		my_error(tr("No key or no Cert and no pkcs12"));
 
-	if (PwDialog::execute(&p, &pass, true) != 1)
+	if (PwDialogCore::execute(&p, &pass, true) != 1)
 		return;
 
 	STACK_OF(X509) *certstack = sk_X509_new_null();
@@ -138,12 +149,62 @@ void pki_pkcs12::writePKCS12(XFile &file) const
 		if (x && x != cert)
 			sk_X509_push(certstack, x->getCert());
 	}
+	int encAlgoNid = encAlgo.getEncAlgoNid();
 	pkcs12 = PKCS12_create(pass.data(), getIntName().toUtf8().data(),
 				key->decryptKey(), cert->getCert(), certstack,
-				0, NID_pbe_WithSHA1And3_Key_TripleDES_CBC,
+				encAlgoNid, encAlgoNid,
 				0, 0, 0);
-	i2d_PKCS12_fp(file.fp(), pkcs12);
+	BioByteArray b;
+	i2d_PKCS12_bio(b, pkcs12);
 	sk_X509_free(certstack);
 	pki_openssl_error();
 	PKCS12_free(pkcs12);
+	file.write(b);
+}
+
+void pki_pkcs12::collect_properties(QMap<QString, QString> &prp) const
+{
+	if (!algorithm.isEmpty())
+		prp["Algorithm"] = algorithm;
+	if (!alias.isEmpty())
+		prp["Friendly Name"] = alias;
+}
+
+// see https://www.rfc-editor.org/rfc/rfc8018 Appendix B.2 for possible encryption schemes
+const QList<int> encAlgo::all_encAlgos(
+	{ NID_pbe_WithSHA1And3_Key_TripleDES_CBC,
+	  NID_aes_256_cbc
+});
+
+int encAlgo::default_encAlgo(NID_pbe_WithSHA1And3_Key_TripleDES_CBC);
+
+encAlgo::encAlgo(int nid) : encAlgo_nid(nid)
+{
+}
+
+encAlgo::encAlgo(const QString &name) : encAlgo_nid(default_encAlgo)
+{
+	QString s(name);
+	encAlgo_nid = OBJ_txt2nid(CCHAR(s.remove(QChar(' '))));
+	ign_openssl_error();
+}
+
+QString encAlgo::name() const
+{
+	return QString(encAlgo_nid == NID_undef ? "" : OBJ_nid2sn(encAlgo_nid));
+}
+
+int encAlgo::getEncAlgoNid() const
+{
+	return encAlgo_nid;
+}
+
+const encAlgo encAlgo::getDefault()
+{
+	return encAlgo(default_encAlgo);
+}
+
+void encAlgo::setDefault(const QString &def)
+{
+	default_encAlgo = encAlgo(def).encAlgo_nid;
 }
