@@ -14,15 +14,11 @@
 #include "pkcs11.h"
 #include "x509name.h"
 #include "func.h"
-#include "db.h"
-#include "widgets/XcaWarning.h"
-#include "widgets/XcaProgress.h"
+#include "XcaProgress.h"
+
+#include "XcaWarningCore.h"
 
 #include <QThread>
-#include <QProgressBar>
-#include <ltdl.h>
-
-#include "openssl_compat.h"
 
 void pki_scard::init(void)
 {
@@ -210,6 +206,27 @@ EVP_PKEY *pki_scard::load_pubkey(pkcs11 &p11, CK_OBJECT_HANDLE object) const
 		EVP_PKEY_assign_EC_KEY(pkey, ec);
 		break;
 	}
+#endif
+#ifdef EVP_PKEY_ED25519
+	case CKK_EC_EDWARDS: {
+		QByteArray ba;
+		ASN1_OCTET_STRING *os;
+		pk11_attr_data grp(CKA_EC_PARAMS);
+		p11.loadAttribute(grp, object);
+		pk11_attr_data pt(CKA_EC_POINT);
+		p11.loadAttribute(pt, object);
+		ba = pt.getData();
+		os = (ASN1_OCTET_STRING *)
+			d2i_bytearray(D2I_VOID(d2i_ASN1_OCTET_STRING), ba);
+		pki_openssl_error();
+		pkey = EVP_PKEY_new_raw_public_key(EVP_PKEY_ED25519, NULL,
+			(const uint8_t *)os->data,
+			os->length);
+		pki_openssl_error();
+		ASN1_OCTET_STRING_free(os);
+		pki_openssl_error();
+		break;
+	}
 #endif
 	default:
 		throw errorEx(QString("Unsupported CKA_KEY_TYPE: %1\n").arg(keytype));
@@ -232,9 +249,7 @@ void pki_scard::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 	pk11_attr_data id(CKA_ID);
 	p11.loadAttribute(id, object);
 	if (id.getAttribute()->ulValueLen > 0) {
-		BIGNUM *cka_id = id.getBignum();
-		object_id = BNoneLine(cka_id);
-		BN_free(cka_id);
+		object_id = QString(id.getData().toHex());
 	}
 
 	try {
@@ -275,9 +290,9 @@ pk11_attr_data pki_scard::getIdAttr() const
 	pk11_attr_data id(CKA_ID);
 	if (object_id.isEmpty())
 		return id;
-	BIGNUM *bn = NULL;
-	BN_hex2bn(&bn, CCHAR(object_id));
-	id.setBignum(bn, true);
+
+	QByteArray val = QByteArray::fromHex(object_id.toLocal8Bit());
+	id.setValue(reinterpret_cast<const unsigned char*>(val.constData()), val.length());
 	return id;
 }
 
@@ -293,10 +308,10 @@ void pki_scard::deleteFromToken()
 pk11_attlist pki_scard::objectAttributesNoId(EVP_PKEY *pk, bool priv) const
 {
 	QByteArray ba;
-	RSA *rsa;
-	DSA *dsa;
+	const RSA *rsa;
+	const DSA *dsa;
 #ifndef OPENSSL_NO_EC
-	EC_KEY *ec;
+	const EC_KEY *ec;
 #endif
 	const BIGNUM *n = NULL;
 	const BIGNUM *e = NULL;
@@ -332,6 +347,12 @@ pk11_attlist pki_scard::objectAttributesNoId(EVP_PKEY *pk, bool priv) const
 		attrs << pk11_attr_ulong(CKA_KEY_TYPE, CKK_EC) <<
 			pk11_attr_data(CKA_EC_PARAMS, ba);
 		break;
+#endif
+#ifdef EVP_PKEY_ED25519
+	case EVP_PKEY_ED25519:
+		attrs << pk11_attr_ulong(CKA_KEY_TYPE, CKK_EC_EDWARDS);
+		// should it also return params, somehow?
+		break;
 #endif
 	default:
 		throw errorEx(QString("Unknown Keytype %d")
@@ -401,10 +422,10 @@ int pki_scard::renameOnToken(const slotid &slot, const QString &name)
 void pki_scard::store_token(const slotid &slot, EVP_PKEY *pkey)
 {
 	QByteArray ba;
-	RSA *rsa;
-	DSA *dsa;
+	const RSA *rsa;
+	const DSA *dsa;
 #ifndef OPENSSL_NO_EC
-	EC_KEY *ec;
+	const EC_KEY *ec;
 #endif
 	pk11_attlist pub_atts;
 	pk11_attlist priv_atts;
@@ -488,7 +509,7 @@ void pki_scard::store_token(const slotid &slot, EVP_PKEY *pkey)
 		pki_openssl_error();
 		size = BN_num_bytes(point);
 		buf = (unsigned char *)OPENSSL_malloc(size);
-		check_oom(buf);
+		Q_CHECK_PTR(buf);
 		BN_bn2bin(point, buf);
 		os = ASN1_OCTET_STRING_new();
 		/* set0 -> ASN1_OCTET_STRING_free() also free()s buf */
@@ -590,10 +611,11 @@ bool pki_scard::find_key_on_card(slotid *slot) const
 	cls << getIdAttr();
 
 	foreach(sl, p11.getSlotList()) {
-		p11.startSession(sl);
+		pkcs11 p11sess;
+		p11sess.startSession(sl);
 
-		foreach(CK_OBJECT_HANDLE object, p11.objectList(cls)) {
-			EVP_PKEY *pkey = load_pubkey(p11, object);
+		foreach(CK_OBJECT_HANDLE object, p11sess.objectList(cls)) {
+			EVP_PKEY *pkey = load_pubkey(p11sess, object);
 			bool match = EVP_PKEY_cmp(key, pkey) == 1;
 			EVP_PKEY_free(pkey);
 
@@ -688,42 +710,6 @@ pki_scard::~pki_scard()
 {
 }
 
-void pki_scard::fromData(const unsigned char *p, db_header_t *head )
-{
-	int version, size;
-	void *ptr = NULL;
-
-	size = head->len - sizeof(db_header_t);
-        version = head->version;
-
-	QByteArray ba((const char*)p, size);
-
-	card_serial = db::stringFromData(ba);
-	card_manufacturer = db::stringFromData(ba);
-	card_label = db::stringFromData(ba);
-	slot_label = db::stringFromData(ba);
-	card_model = db::stringFromData(ba);
-	if (version < 2)
-		card_model.clear();
-	object_id  = db::stringFromData(ba);
-	int count      = db::intFromData(ba);
-	mech_list.clear();
-	for (int i=0; i<count; i++)
-		mech_list << db::intFromData(ba);
-
-	d2i(ba);
-
-	if (key)
-		ptr = EVP_PKEY_get0(key);
-
-	if (!ptr)
-		throw errorEx(tr("Ignoring unsupported token key"));
-
-	if (ba.count() > 0) {
-		my_error(tr("Wrong Size %1").arg(ba.count()));
-	}
-}
-
 QString pki_scard::getTypeString(void) const
 {
 	return tr("Token %1").arg(pki_key::getTypeString());
