@@ -6,17 +6,57 @@
  */
 
 #include "pki_temp.h"
+#include "pki_x509.h"
 #include "func.h"
-#include "db.h"
 #include "oid.h"
 #include "exception.h"
-#include "widgets/MainWindow.h"
 #include <QDir>
 #include <QBuffer>
 #include <QDataStream>
+#include <QRegularExpression>
 
 #define TEMPLATE_DS_VERSION (QDataStream::Qt_4_2)
 
+namespace db {
+
+	QByteArray intToData(uint32_t val)
+	{
+		uint32_t v = xhtonl(val);
+		return QByteArray((char*)&v, sizeof(uint32_t));
+	}
+	uint32_t intFromData(QByteArray &ba)
+	{
+		uint32_t ret;
+		if ((unsigned)(ba.size()) < sizeof(uint32_t)) {
+			throw errorEx(QObject::tr("Out of data"));
+		}
+		memcpy(&ret, ba.constData(), sizeof(uint32_t));
+		ba = ba.mid(sizeof(uint32_t));
+		return xntohl(ret);
+	}
+	bool boolFromData(QByteArray &ba)
+	{
+		unsigned char c;
+		if (ba.size() < 1)
+			throw errorEx(QObject::tr("Out of data"));
+
+		c = ba.constData()[0];
+		ba = ba.mid(1);
+		return c ? true : false;
+	}
+	QString stringFromData(QByteArray &ba)
+	{
+		int idx = ba.indexOf('\0');
+
+		if (idx == -1)
+			throw errorEx(QObject::tr("Error finding endmarker of string"));
+
+		QString ret = QString::fromUtf8(ba.constData(), idx);
+		ba = ba.mid(idx+1);
+		return ret;
+	}
+};
+
 const QList<QString> pki_temp::tmpl_keys = {
 	"subAltName",
 	"issAltName",
@@ -48,7 +88,7 @@ const QList<QString> pki_temp::tmpl_keys = {
 };
 
 pki_temp::pki_temp(const pki_temp *pk)
-	:pki_x509name(pk)
+	:pki_x509name(pk->getIntName())
 {
 	pre_defined = false;
 
@@ -220,7 +260,8 @@ extList pki_temp::fromCert(pki_x509super *cert_or_req)
 
 	settings["kuCritical"] = kuCritical ? "1" : "0";
 	fromExtList(&el, NID_ext_key_usage, "eKeyUse");
-	QStringList eKeyUse = settings["eKeyUse"].split(QRegExp(",\\s*"));
+	QStringList eKeyUse = settings["eKeyUse"]
+							.split(QRegularExpression(",\\s*"));
 	settings["ekuCritical"] = "0";
 	if (eKeyUse.contains("critical")) {
 		eKeyUse.removeOne("critical");
@@ -262,20 +303,11 @@ extList pki_temp::fromCert(pki_x509super *cert_or_req)
 	return el;
 }
 
-void pki_temp::fromData(const unsigned char *p, db_header_t *head )
-{
-	int version, size;
-
-	size = head->len - sizeof(db_header_t);
-	version = head->version;
-	fromData(p, size, version);
-}
-
 static QString old_eKeyUse2QString(int old)
 {
 	QStringList sl;
 
-        for (int i = 0; i < extkeyuse_nid.count(); i++) {
+        for (int i = 0; i < extkeyuse_nid.size(); i++) {
 		if (old & (1<<i)) {
 			sl << OBJ_nid2sn(extkeyuse_nid[i]);
 		}
@@ -331,8 +363,8 @@ void pki_temp::old_fromData(const unsigned char *p, int size, int version)
 		settings["noWellDefinedExpDate"] =
 				QString::number(db::boolFromData(ba));
 
-	if (ba.count() > 0) {
-		my_error(tr("Wrong Size %1").arg(ba.count()));
+	if (ba.size() > 0) {
+		my_error(tr("Wrong Size %1").arg(ba.size()));
 	}
 }
 
@@ -385,17 +417,20 @@ void pki_temp::fromData(const unsigned char *p, int size, int version)
 QByteArray pki_temp::toExportData() const
 {
 	QByteArray data, header;
+	BioByteArray b;
+
 	data = toData();
-	header = db::intToData(data.count());
+	header = db::intToData(data.size());
 	header += db::intToData(TMPL_VERSION);
 	header += data;
-	return header;
+	PEM_write_bio(b, PEM_STRING_XCA_TEMPLATE, (char*)"",
+		(unsigned char*)(header.data()), header.size());
+	return b.byteArray();
 }
 
 void pki_temp::writeTemp(XFile &file) const
 {
-	PEM_file_comment(file);
-	file.write(toExportData());
+	file.write(PEM_comment() + toExportData());
 }
 
 void pki_temp::writeDefault(const QString &dirname) const
@@ -405,11 +440,10 @@ void pki_temp::writeDefault(const QString &dirname) const
 	writeTemp(file);
 }
 
-bool pki_temp::pem(BioByteArray &b, int)
+bool pki_temp::pem(BioByteArray &b)
 {
-	QByteArray ba = toExportData();
-	return PEM_write_bio(b, PEM_STRING_XCA_TEMPLATE, (char*)"",
-		(unsigned char*)(ba.data()), ba.size());
+	b += toExportData();
+	return true;
 }
 
 void pki_temp::fromExportData(QByteArray data)
@@ -430,7 +464,7 @@ void pki_temp::try_fload(XFile &file)
 	QByteArray ba = file.read(4096*1024);
 	try {
 		fromPEM_BIO(BioByteArray(ba).ro(), file.fileName());
-	} catch (errorEx &err) {
+	} catch (errorEx &) {
 		fromExportData(ba);
 	}
 	pki_openssl_error();
@@ -442,14 +476,15 @@ void pki_temp::fload(const QString &fname)
 		XFile file(fname);
 		file.open_read();
 		try_fload(file);
-	} catch (errorEx &err) {
 #if defined(Q_OS_WIN32)
+	} catch (errorEx &) {
 		/* Try again in ascii mode on Windows
 		 * to support pre 1.1.0 template exports */
 		XFile file(fname);
 		file.open(QIODevice::ReadOnly | QIODevice::QIODevice::Text);
 		try_fload(file);
 #else
+	} catch (errorEx &err) {
 		throw err;
 #endif
 	}
