@@ -16,8 +16,6 @@
 #include "base.h"
 #include "exception.h"
 #include "pass_info.h"
-#include "openssl_compat.h"
-#include "widgets/XcaWarning.h"
 
 pki_x509::pki_x509(X509 *c)
 	:pki_x509super()
@@ -174,7 +172,7 @@ QSqlError pki_x509::deleteSqlData()
 pki_x509 *pki_x509::findIssuer()
 {
 	XSqlQuery q;
-	pki_x509 *issuer;
+	pki_x509 *issuer = NULL;
 	unsigned hash;
 
 	if ((issuer = getSigner()) != NULL)
@@ -191,23 +189,31 @@ pki_x509 *pki_x509::findIssuer()
 	q.bindValue(0, hash);
 	q.exec();
 	while (q.next()) {
-		issuer = Store.lookupPki<pki_x509>(q.value(0));
-		if (!issuer) {
+		pki_x509 *an_issuer = Store.lookupPki<pki_x509>(q.value(0));
+		qDebug() << "Possibvle Issuer of" << *this << *an_issuer << an_issuer->getNotAfter();
+		if (!an_issuer) {
 			qDebug("Certificate with id %d not found",
                                 q.value(0).toInt());
+			continue;
 		}
-		if (verify(issuer)) {
-			return issuer;
+		if (verify_only(an_issuer)) {
+			if (!issuer || (issuer->getNotAfter() < an_issuer->getNotAfter())) {
+				qDebug() << "New issuer of" << *this << *an_issuer << an_issuer->getNotAfter();
+				issuer = an_issuer;
+			}
 		}
 	}
-	return NULL;
+	verify(issuer);
+	return issuer;
 }
 
 void pki_x509::fromPEM_BIO(BIO *bio, const QString &fname)
 {
 	X509 *_cert;
 	_cert = PEM_read_bio_X509(bio, NULL, NULL, NULL);
-	openssl_error(fname);
+	openssl_error_msg(fname);
+	if (!_cert)
+		throw errorEx();
 	X509_free(cert);
 	cert = _cert;
 }
@@ -217,11 +223,12 @@ void pki_x509::fload(const QString &fname)
 	X509 *_cert;
 	XFile file(fname);
 	file.open_read();
-	_cert = PEM_read_X509(file.fp(), NULL, NULL, NULL);
+	QByteArray ba(file.readAll());
+
+	_cert = PEM_read_bio_X509(BioByteArray(ba).ro(), NULL, NULL, NULL);
 	if (!_cert) {
 		pki_ign_openssl_error();
-		file.retry_read();
-		_cert = d2i_X509_fp(file.fp(), NULL);
+		_cert = d2i_X509_bio(BioByteArray(ba).ro(), NULL);
 	}
 	if (pki_ign_openssl_error() || !_cert) {
 		if (_cert)
@@ -563,7 +570,7 @@ bool pki_x509::hasExtension(int nid) const
 	return getV3ext().idxByNid(nid) != -1;
 }
 
-void pki_x509::sign(pki_key *signkey, const EVP_MD *digest)
+void pki_x509::sign(pki_key *signkey, const digest &digest)
 {
 	EVP_PKEY *tkey;
 	if (!signkey) {
@@ -571,77 +578,12 @@ void pki_x509::sign(pki_key *signkey, const EVP_MD *digest)
 	}
 	tkey = signkey->decryptKey();
 	pki_openssl_error();
-	X509_sign(cert, tkey, digest);
+	X509_sign(cert, tkey, digest.MD());
 	pki_openssl_error();
 	EVP_PKEY_free(tkey);
 	pki_openssl_error();
 }
 
-void pki_x509::fromData(const unsigned char *p, db_header_t *head)
-{
-	int version, size;
-	bool isRevoked = false;
-
-	version = head->version;
-	size = head->len - sizeof(db_header_t);
-
-	QByteArray ba((const char*)p, size);
-
-	d2i(ba);
-	pki_openssl_error();
-	/* trust = */ db::intFromData(ba);
-	if (version < 4) {
-		a1time revoked;
-		isRevoked = db::boolFromData(ba);
-		revoked.d2i(ba);
-		pki_openssl_error();
-		if (isRevoked) {
-			revocation.setDate(revoked);
-			revocation.setSerial(getSerial());
-		}
-	}
-	pki_openssl_error();
-	/* Superflous CaSerial = */db::stringFromData(ba);
-	QString caTemplate = db::stringFromData(ba);
-	crlDays = db::intFromData(ba);
-	crlExpire.d2i(ba);
-	pki_openssl_error();
-	if (version > 1)
-		/* randomSerial = */ db::boolFromData(ba);
-	if (version > 2)
-		crlNumber.setHex(db::stringFromData(ba));
-	pki_openssl_error();
-	if (version > 2 && version < 4) {
-		// load own revocation info, to tell daddy about it
-		a1time invalDate;
-		QString revoke_reason = db::stringFromData(ba);
-		invalDate.d2i(ba);
-		pki_openssl_error();
-		if (isRevoked) {
-			revocation.setReason(revoke_reason);
-			revocation.setInvalDate(invalDate);
-		}
-	}
-	pki_openssl_error();
-	if (version > 3) {
-		fromDataRevList.fromBA(ba);
-		pki_openssl_error();
-	}
-	if (ba.count() > 0) {
-		my_error(tr("Wrong Size %1").arg(ba.count()));
-	}
-	pki_openssl_error();
-
-	XSqlQuery q;
-	SQL_PREPARE(q, "SELECT id FROM items WHERE name=? AND type=?");
-	q.bindValue(0, caTemplate);
-	q.bindValue(1, tmpl);
-	q.exec();
-	if (q.next())
-		caTemplateSqlId = q.value(0);
-}
-
-
 void pki_x509::writeDefault(const QString &dirname) const
 {
 	XFile file(get_dump_filename(dirname, ".crt"));
@@ -653,13 +595,15 @@ void pki_x509::writeCert(XFile &file, bool PEM) const
 {
 	if (!cert)
 		return;
+	BioByteArray b;
 	if (PEM) {
-		PEM_file_comment(file);
-		PEM_write_X509(file.fp(), cert);
+		b += PEM_comment();
+		PEM_write_bio_X509(b, cert);
 	} else {
-		i2d_X509_fp(file.fp(), cert);
+		i2d_X509_bio(b, cert);
 	}
 	pki_openssl_error();
+	file.write(b);
 }
 
 QString pki_x509::getIndexEntry()
@@ -681,7 +625,7 @@ QString pki_x509::getIndexEntry()
 		QString(X509_NAME_oneline(getSubject().get(), NULL, 0)));
 }
 
-bool pki_x509::pem(BioByteArray &b, int)
+bool pki_x509::pem(BioByteArray &b)
 {
 	return PEM_write_bio_X509(b, cert);
 }
@@ -955,7 +899,7 @@ QStringList pki_x509::icsVEVENT() const
 			.arg(getIntName())
 			.arg(getNotBefore().toPretty())
 			.arg(getNotAfter().toPretty())
-			.arg(Database.name())
+			.arg(nativeSeparator(Database.name()))
 	);
 }
 
@@ -1058,10 +1002,6 @@ bool pki_x509::visible() const
 
 QVariant pki_x509::bg_color(const dbheader *hd) const
 {
-#define BG_RED     QBrush(QColor(255,  0,  0))
-#define BG_YELLOW  QBrush(QColor(255,255,  0))
-#define BG_CYAN    QBrush(QColor(127,255,212))
-
 	if (Settings["no_expire_colors"])
 		return QVariant();
 
@@ -1087,15 +1027,15 @@ QVariant pki_x509::bg_color(const dbheader *hd) const
 	switch (hd->id) {
 		case HD_cert_notBefore:
 			if (nb > now || !nb.isValid() || nb.isUndefined())
-				return QVariant(BG_RED);
+				return QVariant(red);
 			break;
 		case HD_cert_notAfter: {
 			if (na.isUndefined())
-				return QVariant(BG_CYAN);
+				return QVariant(cyan);
 			if (na < now)
-				return QVariant(BG_RED);
+				return QVariant(red);
 			if (certwarn < now)
-				return QVariant(BG_YELLOW);
+				return QVariant(yellow);
 			break;
 		}
 		case HD_cert_crl_expire:
@@ -1105,9 +1045,9 @@ QVariant pki_x509::bg_color(const dbheader *hd) const
 				if (!crlExpire.isUndefined()) {
 					crlwarn = crlex.addSecs(-2 *60*60*24);
 					if (crlex < now)
-						return QVariant(BG_RED);
+						return QVariant(red);
 					if (crlwarn < now || !crlex.isValid())
-						return QVariant(BG_YELLOW);
+						return QVariant(yellow);
 				}
 			}
 	}
