@@ -14,6 +14,7 @@
 #include "pki_x509super.h"
 #include "x509v3ext.h"
 #include "x509name.h"
+#include "digest.h"
 
 #define VIEW_x509req_request 7
 #define VIEW_x509req_signed 8
@@ -40,7 +41,6 @@ class pki_x509req : public pki_x509super
 		void fromPEM_BIO(BIO *bio, const QString &name);
 		void fload(const QString &fname);
 		void writeDefault(const QString &dirname) const;
-		void fromData(const unsigned char *p, db_header_t *head);
 		x509name getSubject() const;
 		void writeReq(XFile &file, bool pem) const;
 		void markSigned(bool signe);
@@ -56,7 +56,7 @@ class pki_x509req : public pki_x509super
 		bool verify() const;
 		pki_key *getPubKey() const;
 		void createReq(pki_key *key, const x509name &dn,
-				const EVP_MD *md, extList el);
+				const digest &digest, extList el);
 		void setSubject(const x509name &n);
 		QVariant column_data(const dbheader *hd) const;
 		QVariant getIcon(const dbheader *hd) const;
@@ -75,7 +75,7 @@ class pki_x509req : public pki_x509super
 		virtual QString getMsg(msg_type msg) const;
 		void d2i(QByteArray &ba);
 		QByteArray i2d() const;
-		bool pem(BioByteArray &, int);
+		bool pem(BioByteArray &);
 		bool visible() const;
 		QSqlError insertSqlData();
 		QSqlError deleteSqlData();
