@@ -1,8 +1,11 @@
 #include "settings.h"
 #include "func.h"
 #include "sql.h"
-#include "widgets/hashBox.h"
-#include "widgets/NewKey.h"
+#include "pki_key.h"
+#include "digest.h"
+#include "pki_pkcs12.h"
+#include "pki_export.h"
+
 #include <QDir>
 #include <QFile>
 #include <openssl/asn1.h>
@@ -30,13 +33,15 @@ settings::settings()
 	defaul["mandatory_dn"] = "";
 	defaul["explicit_dn"] = "C,ST,L,O,OU,CN,emailAddress";
 	defaul["string_opt"] = "MASK:0x2002";
-	defaul["workingdir"] = getHomeDir() + "/";
-	defaul["default_hash"] = hashBox::getDefault();
+	defaul["workingdir"] = "";
+	defaul["default_hash"] = digest::getDefault().name();
+	defaul["pkcs12_enc_algo"] = encAlgo::getDefault().name();
 	defaul["ical_expiry"] = "1W";
 	defaul["cert_expiry"] = "80%";
 	defaul["serial_len"] = "64";
 	defaul["fp_separator"] = ":";
 	defaul["fp_digits"] = "2";
+	defaul["KeyFormat"] = QString::number(DEFAULT_KEY_CLIPBOARD_TYPE);
 
 	hostspecific << "pkcs11path" << "workingdir" << "mw_geometry";
 }
@@ -55,9 +60,11 @@ void settings::setAction(const QString &key, const QString &value)
 	if (key == "string_opt")
 		ASN1_STRING_set_default_mask_asc((char*)CCHAR(value));
 	else if (key == "default_hash")
-		hashBox::setDefault(value);
+		digest::setDefault(value);
+	else if (key == "pkcs12_enc_algo")
+		encAlgo::setDefault(value);
 	else if (key == "defaultkey")
-		NewKey::defaultjob = keyjob(value);
+		keyjob::defaultjob = keyjob(value);
 	else if (key == "optionflags") {
 		XSqlQuery q;
 		Transaction;
@@ -97,6 +104,8 @@ void settings::load_settings()
 				continue; // Skip non-existing working-dir
 			if (!value.isEmpty() && !value.endsWith("/"))
 				value += "/";
+			if (value.isEmpty())
+				value = getHomeDir() + "/";
 		}
 		db_keys << key;		// Key with host ID
 		setAction(l[0], value);	// Key without host ID
