@@ -6,11 +6,12 @@
  */
 
 #include <QString>
+#include <QRegularExpression>
 #include <QDebug>
 #include "base.h"
 #include "sql.h"
 #include "settings.h"
-#include "widgets/XcaWarning.h"
+#include "XcaWarningCore.h"
 
 int DbTransaction::mutex;
 int DbTransaction::error;
@@ -141,7 +142,7 @@ QString XSqlQuery::rewriteQuery(QString _q)
 
 	QString m = tables.join("|") + "|i_" + tables.join("|i_");
 	m = QString("\\b(%1)").arg(m);
-	query = query.replace(QRegExp(m), table_prefix + "\\1");
+	query = query.replace(QRegularExpression(m), table_prefix + "\\1");
 
 	return query;
 }
@@ -149,7 +150,11 @@ QString XSqlQuery::rewriteQuery(QString _q)
 QString XSqlQuery::query_details()
 {
 	QString lq = lastq;
-	QList<QVariant> list = boundValues().values();
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+	QVariantList list = boundValues();
+#else
+	QVariantList list = boundValues().values();
+#endif
 	QStringList sl;
 
 	if (query != lastq) {
@@ -171,11 +176,7 @@ QSqlError XSqlQuery::lastError()
 	return QSqlError(QString("%1 - %2").arg(e.driverText())
 					.arg(query_details()),
 			 e.databaseText(), e.type(),
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
 			 e.nativeErrorCode()
-#else
-			 e.number()
-#endif
 			);
 }
 
