@@ -19,22 +19,24 @@ class CertDetail: public QDialog, public Ui::CertDetail
 		Q_OBJECT
 
 		bool showConf;
-		QVariant keySqlId, issuerSqlId;
+		QVariant keySqlId, issuerSqlId, thisSqlId;
 		QString conf, exts;
 		QLabel *labelFromAsn1String(ASN1_STRING *s);
-		void setX509super(pki_x509super *x);
-		pki_key *myPubKey;
-
-	public:
-		CertDetail(QWidget *parent);
-		~CertDetail();
+		pki_key *myPubKey, *tmpPubKey;
 		void setCert(pki_x509 *cert);
 		void setReq(pki_x509req *req);
 
+	public:
+		CertDetail(QWidget *w = nullptr);
+		~CertDetail();
+		void setX509super(pki_x509super *x);
+		static void showCert(QWidget *parent, pki_x509super *x);
+
 	private slots:
 		void on_showExt_clicked();
 		void itemChanged(pki_base *pki);
 		void showPubKey();
+		void showIssuer();
 };
 
 #endif
