@@ -6,20 +6,42 @@
  */
 
 #include "lib/pki_scard.h"
+#include "lib/load_obj.h"
 #include "KeyTreeView.h"
 #include "MainWindow.h"
 #include "KeyDetail.h"
 #include "NewKey.h"
+#include "ExportDialog.h"
 #include "XcaWarning.h"
 #include <QAbstractItemView>
+#include <QActionGroup>
 #include <QMenu>
 
 void KeyTreeView::fillContextMenu(QMenu *menu, QMenu *subExport,
 			const QModelIndex &index, QModelIndexList indexes)
 {
+	QMenu *clipboard;
+	QAction *a;
 	bool multi = indexes.size() > 1;
+	QActionGroup *group = new QActionGroup(menu);
 
 	pki_key *key = db_base::fromIndex<pki_key>(index);
+	int exp_type = Settings["KeyFormat"];
+	const pki_export *x;
+
+	clipboard = menu->addMenu(tr("Clipboard format"));
+	foreach(x, pki_export::select(asym_key, 0)) {
+		if (!(x->flags & F_CLIPBOARD))
+			continue;
+		a = clipboard->addAction(x->desc);
+		a->setData(x->id);
+		a->setCheckable(true);
+		a->setChecked(exp_type == x->id);
+		group->addAction(a);
+	}
+
+	connect(group, SIGNAL(triggered(QAction*)),
+		this, SLOT(clipboardFormat(QAction*)));
 
 	if (indexes.size() == 0 || !key)
 		return;
@@ -186,3 +208,28 @@ void KeyTreeView::newItem(const QString &name)
 		keys()->newKey(dlg->getKeyJob(), dlg->keyDesc->text());
 	delete dlg;
 }
+
+void KeyTreeView::clipboardFormat(QAction *a)
+{
+	Settings["KeyFormat"] = a->data().toInt();
+}
+
+void KeyTreeView::load(void)
+{
+	load_key l;
+	load_default(&l);
+}
+
+ExportDialog *KeyTreeView::exportDialog(const QModelIndexList &indexes)
+{
+	if (indexes.size() == 0)
+		return NULL;
+	pki_key *key = db_base::fromIndex<pki_key>(indexes[0]);
+	return new ExportDialog(this,
+		tr("Export public key [%1]").arg(key->getTypeString()),
+		tr("Private Keys ( *.pem *.der *.pk8 );; "
+		   "SSH Public Keys ( *.pub )"), indexes,
+		QPixmap(key->isToken() ? ":scardImg" : ":keyImg"),
+		pki_export::select(asym_key, basemodel->exportFlags(indexes)),
+		"keyexport");
+}
