@@ -11,6 +11,7 @@
 #include "XcaTreeView.h"
 #include "lib/db_key.h"
 
+class QAction;
 class KeyTreeView: public XcaTreeView
 {
 	Q_OBJECT
@@ -25,6 +26,7 @@ class KeyTreeView: public XcaTreeView
 	void fillContextMenu(QMenu *menu, QMenu *subExport,
 			const QModelIndex &index, QModelIndexList indexes);
 	void showPki(pki_base *pki);
+	ExportDialog *exportDialog(const QModelIndexList &indexes);
 
    public slots:
 	void resetOwnPass();
@@ -34,6 +36,8 @@ class KeyTreeView: public XcaTreeView
 	void changeSoPin();
 	void toToken();
 	void newItem();
+	void load();
 	void newItem(const QString &name);
+	void clipboardFormat(QAction*);
 };
 #endif
