@@ -6,11 +6,10 @@
  */
 
 
-//#define MDEBUG
 #include "MainWindow.h"
 #include "XcaApplication.h"
 #include "ImportMulti.h"
-#include "dhgen.h"
+#include "hashBox.h"
 
 #include <QApplication>
 #include <QClipboard>
@@ -27,16 +26,22 @@
 
 #include <openssl/err.h>
 
+#include "lib/entropy.h"
 #include "lib/Passwd.h"
 #include "lib/database_model.h"
 #include "lib/exception.h"
 #include "lib/pki_evp.h"
 #include "lib/pki_multi.h"
 #include "lib/pki_scard.h"
+#include "lib/dhgen.h"
+#include "lib/load_obj.h"
+
 #include "XcaDialog.h"
 #include "XcaWarning.h"
+#include "XcaProgressGui.h"
 #include "PwDialog.h"
 #include "OpenDb.h"
+#include "Help.h"
 #include "OidResolver.h"
 
 OidResolver *MainWindow::resolver = NULL;
@@ -82,11 +87,7 @@ MainWindow::MainWindow() : QMainWindow()
 	wdList << keyButtons << reqButtons << certButtons <<
 		tempButtons <<	crlButtons;
 
-	QStringList drivers = QSqlDatabase::drivers();
-	foreach(QString driver, drivers) {
-		QSqlDatabase d = QSqlDatabase::addDatabase(driver, driver +"_C");
-		qDebug() << "DB driver:" << driver;
-	}
+	OpenDb::initDatabases();
 
 	historyMenu = NULL;
 	helpdlg = new Help();
@@ -96,12 +97,6 @@ MainWindow::MainWindow() : QMainWindow()
 	init_images();
 	homedir = getHomeDir();
 
-#ifdef MDEBUG
-	CRYPTO_malloc_debug_init();
-	CRYPTO_mem_ctrl(CRYPTO_MEM_CHECK_ON);
-	qWarning() << "malloc() debugging on.";
-#endif
-
 	ERR_load_crypto_strings();
 	OpenSSL_add_all_algorithms();
 
@@ -125,14 +120,17 @@ MainWindow::MainWindow() : QMainWindow()
 
 	views << keyView << reqView << certView << crlView << tempView;
 
+	pki_base::setupColors(palette());
+
 	foreach(XcaTreeView *v, views)
 		v->setMainwin(this, searchEdit);
 
-	dhgen = NULL;
-	dhgenBar = new QProgressBar();
-	check_oom(dhgenBar);
-	dhgenBar->setMinimum(0);
-	dhgenBar->setMaximum(0);
+	XcaProgress::setGui(new XcaProgressGui(this));
+	xcaWarning::setGui(new xcaWarningGui());
+	PwDialogCore::setGui(new PwDialogUI());
+
+	dhgen = nullptr;
+	dhgenProgress = nullptr;
 }
 
 void MainWindow::dropEvent(QDropEvent *event)
@@ -213,9 +211,7 @@ void MainWindow::init_images()
 void MainWindow::loadPem()
 {
 	load_pem l;
-	db_key *keys = Database.model<db_key>();
-	if (keys)
-		keys->load_default(l);
+	keyView->load_default(&l);
 }
 
 bool MainWindow::pastePem(QString text, bool silent)
@@ -290,11 +286,11 @@ void MainWindow::initToken()
 			arg(slotname) + "\n" + ti.pinInfo());
 		p.setPin();
 		if (ti.tokenInitialized()) {
-			ret = PwDialog::execute(&p, &pin, false);
+			ret = PwDialogCore::execute(&p, &pin, false);
 		} else {
 			p.setDescription(tr("Please enter the new SO PIN (PUK) for the token '%1'").
 			arg(slotname) + "\n" + ti.pinInfo());
-			ret = PwDialog::execute(&p, &pin, true);
+			ret = PwDialogCore::execute(&p, &pin, true);
 		}
 		if (ret != 1)
 			return;
@@ -426,10 +422,8 @@ MainWindow::~MainWindow()
 	OBJ_cleanup();
 	delete dbindex;
 	delete helpdlg;
-#ifdef MDEBUG
-	fprintf(stderr, "Memdebug:\n");
-	CRYPTO_mem_leaks_fp(stderr);
-#endif
+
+	XcaProgress::setGui(nullptr);
 }
 
 void MainWindow::closeEvent(QCloseEvent *e)
@@ -459,7 +453,7 @@ int MainWindow::checkOldGetNewPass(Passwd &pass)
 		/* Try empty password */
 		if (pki_evp::sha512passwT(pass, passHash) != passHash) {
 			/* Not the empty password, check it */
-			if (PwDialog::execute(&p, &pass, false) != 1)
+			if (PwDialogCore::execute(&p, &pass, false) != 1)
 				return 0;
 		}
 
@@ -473,7 +467,7 @@ int MainWindow::checkOldGetNewPass(Passwd &pass)
 			"to encrypt your private keys in the database-file"),
 			this);
 
-	return PwDialog::execute(&p, &pass, true) != 1 ? 0 : 1;
+	return PwDialogCore::execute(&p, &pass, true) != 1 ? 0 : 1;
 }
 
 void MainWindow::changeDbPass()
@@ -577,6 +571,13 @@ enum open_result MainWindow::init_database(const QString &name,
 	return pw_ok;
 }
 
+void MainWindow::showDatabaseName()
+{
+	if (Database.isOpen())
+		dbindex->setText(tr("Database: %1")
+			.arg(compressFilename(Database.name())));
+}
+
 enum open_result MainWindow::setup_open_database()
 {
 	if (!Database.isOpen())
@@ -586,8 +587,7 @@ enum open_result MainWindow::setup_open_database()
 		homedir = QFileInfo(Database.name()).canonicalPath();
 
 	setItemEnabled(true);
-	dbindex->setText(tr("Database") + ": " +
-			 compressFilename(Database.name()));
+	showDatabaseName();
 	set_geometry(Settings["mw_geometry"]);
 
 	if (pki_evp::passwd.isNull())
@@ -595,9 +595,9 @@ enum open_result MainWindow::setup_open_database()
 
 	enableTokenMenu(pkcs11::libraries.loaded());
 
-	hashBox hb(this);
-	if (hb.isInsecure()) {
-		XCA_WARN(tr("The currently used default hash '%1' is insecure. Please select at least 'SHA 224' for security reasons.").arg(hb.currentHashName()));
+	digest defdig(digest::getDefault());
+	if (defdig.isInsecure()) {
+		XCA_WARN(tr("The currently used default hash '%1' is insecure. Please select at least 'SHA 224' for security reasons.").arg(defdig.name()));
 		setOptions();
 	}
 
@@ -677,8 +677,7 @@ void MainWindow::exportIndex(const QString &fname, bool hierarchy) const
 
 void MainWindow::generateDHparamDone()
 {
-	statusBar()->removeWidget(dhgenBar);
-	errorEx e(dhgen->error);
+	errorEx e(dhgen->error());
 	if (e.isEmpty())
 		XCA_INFO(tr("Diffie-Hellman parameters saved as: %1")
 			.arg(dhgen->filename()));
@@ -686,6 +685,8 @@ void MainWindow::generateDHparamDone()
 		XCA_ERROR(e);
 	dhgen->deleteLater();
 	dhgen = NULL;
+	delete dhgenProgress;
+	dhgenProgress = nullptr;
 }
 
 void MainWindow::generateDHparam()
@@ -715,9 +716,7 @@ void MainWindow::generateDHparam()
 		if (fname == "")
 			throw errorEx("");
 		dhgen = new DHgen(fname, bits);
-		check_oom(dhgen);
-		statusBar()->addPermanentWidget(dhgenBar, 1);
-		dhgenBar->show();
+		dhgenProgress = new XcaProgress(QString("Diffie-Hellman"), 0);
 		dhgen->start(QThread::LowestPriority);
 		connect(dhgen, SIGNAL(finished()),
 			this, SLOT(generateDHparamDone()));
@@ -735,9 +734,7 @@ void MainWindow::changeEvent(QEvent *event)
 		foreach(db_base *model, Database.getModels())
 			model->updateHeaders();
 
-		if (Database.isOpen())
-			dbindex->setText(tr("Database") + ": " +
-					Database.name());
+		showDatabaseName();
 		searchEdit->setPlaceholderText(tr("Search"));
 	}
 	QMainWindow::changeEvent(event);
