@@ -17,6 +17,8 @@
 #include <QPixmap>
 #include <QPushButton>
 #include <QValidator>
+#include <QRegularExpression>
+#include <QRegularExpressionValidator>
 #include <QMap>
 #include <QPair>
 
@@ -66,7 +68,8 @@ QList<nameEdit> NewX509::setupExplicitInputs(NIDlist nid_list,
 		layout = new QGridLayout(parent);
 		layout->setAlignment(Qt::AlignTop);
 		layout->setSpacing(6);
-		layout->setMargin(0);
+		layout->setContentsMargins(0, 0, 0, 0);
+		layout->setContentsMargins(11, 11, 11, 11);
 	}
 	int n = 0, col = 0;
 
@@ -107,7 +110,7 @@ QList<nameEdit> NewX509::setupExplicitInputs(NIDlist nid_list,
 	return edits;
 }
 
-NewX509::NewX509(QWidget *w) : QDialog(w ?: mainwin)
+NewX509::NewX509(QWidget *w) : QDialog(w ? w : mainwin)
 {
 	int i;
 	QStringList keys;
@@ -128,7 +131,7 @@ NewX509::NewX509(QWidget *w) : QDialog(w ?: mainwin)
 	tabnames = QStringList({
 		"wizard_src", "wizard_subject", "wizard_extensions",
 		"wizard_keyusage", "wizard_netscape", "wizard_advanced",
-		"comment"});
+		"wizard_comment"});
 	extDNlist->setKeys(keys);
 	extDNlist->setInfoLabel(extDNinfo);
 	connect(extDNlist->itemDelegateForColumn(1),
@@ -206,6 +209,7 @@ NewX509::NewX509(QWidget *w) : QDialog(w ?: mainwin)
 	attrWidget->hide();
 	pt = none;
 	notAfter->setEndDate(true);
+	basicPath->setValidator(new QIntValidator(0, 1000, this));
 
 	QMap<int, QWidget*> nidWidget;
 	nidWidget[NID_subject_alt_name] = sanLbl;
@@ -338,8 +342,8 @@ void NewX509::setupLineEditByNid(int nid, QLineEdit *l)
 			info << tr("maximum size: %1").arg(tab->maxsize);
 		if (tab->mask == B_ASN1_PRINTABLESTRING) {
 			info << tr("only a-z A-Z 0-9 '()+,-./:=?");
-			QRegExp rx("[a-zA-Z0-9'()+,-./:=?]+");
-			validator = new QRegExpValidator(rx, this);
+			QRegularExpression rx("[a-zA-Z0-9'()+,-./:=?]+");
+			validator = new QRegularExpressionValidator(rx, this);
 		} else if (tab->mask == B_ASN1_IA5STRING) {
 			info << tr("only 7-bit clean characters");
 		}
@@ -416,7 +420,7 @@ void NewX509::fromX509super(pki_x509super *cert_or_req, bool applyTemp)
 		usedKeysToo->setChecked(true);
 		keyList->setCurrentPkiItem(key);
 	}
-	hashAlgo->setCurrentMD(cert_or_req->getDigest());
+	hashAlgo->setCurrent(cert_or_req->getDigest());
 
 	switch(cert_or_req->getType()) {
 	case x509: {
@@ -611,12 +615,12 @@ void NewX509::on_reqSubChange_clicked()
 	on_fromReqCB_clicked();
 }
 
-void NewX509::on_keyList_currentIndexChanged(const QString &)
+void NewX509::on_keyList_currentIndexChanged(int)
 {
 	switchHashAlgo();
 }
 
-void NewX509::on_reqList_currentIndexChanged(const QString &)
+void NewX509::on_reqList_currentIndexChanged(int)
 {
 	switchHashAlgo();
 }
@@ -635,13 +639,10 @@ void NewX509::switchHashAlgo()
 
 	key = sig ? sig->getRefKey() : getSelectedKey();
 
-	if (key) {
-		hashAlgo->setKeyType(key->getKeyType());
+	if (key)
 		hashAlgo->setupHashes(key->possibleHashNids());
-	} else {
-		hashAlgo->setKeyType(EVP_PKEY_RSA);
+	else
 		hashAlgo->setupAllHashes();
-	}
 }
 
 void NewX509::on_showReqBut_clicked()
@@ -938,6 +939,7 @@ void NewX509::on_adv_validate_clicked()
 
 void NewX509::checkIcon(const QString &text, int nid, QLabel *img)
 {
+	x509v3ext ext;
 	if (text.isEmpty()) {
 		img->clear();
 		return;
@@ -945,23 +947,19 @@ void NewX509::checkIcon(const QString &text, int nid, QLabel *img)
 	ign_openssl_error();
 	switch (nid) {
 	case NID_subject_alt_name:
-		getSubAltName();
+		ext = getSubAltName();
 		break;
 	case NID_issuer_alt_name:
-		getIssAltName();
+		ext = getIssAltName();
 		break;
 	case NID_crl_distribution_points:
-		getCrlDist();
+		ext = getCrlDist();
 		break;
 	case NID_info_access:
-		getAuthInfAcc();
+		ext = getAuthInfAcc();
 		break;
 	}
-	if (ign_openssl_error()) {
-		img->setPixmap(QPixmap(":warnIco"));
-		return;
-	}
-	img->setPixmap(QPixmap(":doneIco"));
+	img->setPixmap(ext.isValid() ? QPixmap(":doneIco") : QPixmap(":warnIco"));
 }
 
 void NewX509::checkSubAltName(const QString & text)
@@ -1082,8 +1080,26 @@ int NewX509::validateExtensions(QString nconf, QString &result)
 		errtxt += "</ul>\n<hr>\n";
 		result = errtxt + result;
 	}
+	QString lineext;
+	if (!subAltName->text().isEmpty() && !getSubAltName().isValid())
+		lineext += "The Subject Alternative Name is invalid<br>\n";
+	if (!issAltName->text().isEmpty() && !getIssAltName().isValid())
+		lineext += "The Issuer Alternative Name is invalid<br>\n";
+	if (!crlDist->text().isEmpty() && !getCrlDist().isValid())
+		lineext += "The CRL Distribution Point is invalid<br>\n";
+	if (!authInfAcc->text().isEmpty() && !getAuthInfAcc().isValid())
+		lineext += "The Authority Information Access is invalid<br>\n";
+	if (!lineext.isEmpty()) {
+		if (!result.isEmpty())
+			result += "\n<hr>\n";
+		result += lineext;
+		ret = 3;
+	}
+	if (ret == 0 && ext_count == 0 && pt == x509)
+		ret = 2;
+
 	ign_openssl_error();
-	return ret == 1 ? 1 : ext_count == 0 && pt == x509 ? 2 : 0;
+	return ret;
 }
 
 void NewX509::on_editSubAlt_clicked()
@@ -1150,7 +1166,7 @@ QString NewX509::mandatoryDnRemain()
 void NewX509::gotoTab(int tab)
 {
 	for (int i=0; i<tabWidget->count(); i++) {
-		if (tabWidget->tabText(i) == tabnames[tab]) {
+		if (tabWidget->widget(i)->objectName() == tabnames[tab]) {
 			tabWidget->setCurrentIndex(i);
 			break;
 		}
@@ -1170,7 +1186,7 @@ void NewX509::accept()
 		xn = getX509name(1);
 	} catch (errorEx &err) {
 		gotoTab(1);
-		xcaWarning msg(this, err.getString());
+		xcaWarningBox msg(this, err.getString());
 		msg.addButton(QMessageBox::Ok);
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		if (msg.exec() == QMessageBox::Close) {
@@ -1183,7 +1199,7 @@ void NewX509::accept()
 		gotoTab(1);
 		lenErr = tr("The following length restrictions of RFC3280 are violated:") +
 			"\n" + lenErr;
-		xcaWarning msg(this, lenErr);
+		xcaWarningBox msg(this, lenErr);
 		msg.addButton(QMessageBox::Ok, tr("Edit subject"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
@@ -1201,7 +1217,7 @@ void NewX509::accept()
 	}
 	if (fromReqCB->isChecked() && !getSelectedReq()->verify()) {
 		gotoTab(0);
-		xcaWarning msg(this,
+		xcaWarningBox msg(this,
 			tr("The verification of the Certificate request failed.\nThe rollout should be aborted."));
 		msg.addButton(QMessageBox::Ok, tr("Continue anyway"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
@@ -1213,7 +1229,7 @@ void NewX509::accept()
 		QString cn = getX509name().getMostPopular();
 		if (cn.isEmpty()) {
 			gotoTab(1);
-			xcaWarning msg(this,
+			xcaWarningBox msg(this,
 				tr("The internal name and the common name are empty.\nPlease set at least the internal name."));
 			msg.addButton(QMessageBox::Ok, tr("Edit name"));
 			msg.addButton(QMessageBox::Close, tr("Abort rollout"));
@@ -1229,7 +1245,7 @@ void NewX509::accept()
 				!fromReqCB->isChecked())
 	{
 		gotoTab(1);
-		xcaWarning msg(this,
+		xcaWarningBox msg(this,
 			tr("There is no Key selected for signing."));
 		msg.addButton(QMessageBox::Ok, tr("Select key"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
@@ -1244,7 +1260,7 @@ void NewX509::accept()
 	if (!unsetDN.isEmpty()) {
 		gotoTab(1);
 		QString text = tr("The following distinguished name entries are empty:\n%1\nthough you have declared them as mandatory in the options menu.").arg(unsetDN);
-		xcaWarning msg(this, text);
+		xcaWarningBox msg(this, text);
 		msg.addButton(QMessageBox::Ok, tr("Edit subject"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
@@ -1276,7 +1292,7 @@ void NewX509::accept()
 	if ((!signkey || signkey->isPubKey()) && pt != tmpl) {
 		QString txt;
 		gotoTab(signer ? 0 : 1);
-		xcaWarning msg(this,
+		xcaWarningBox msg(this,
 			tr("The key you selected for signing is not a private one."));
 		txt = signer ? tr("Select other signer"):tr("Select other key");
 		msg.addButton(QMessageBox::Ok, txt);
@@ -1286,9 +1302,9 @@ void NewX509::accept()
 		}
 		return;
         }
-	if (hashAlgo->count() > 0 && hashAlgo->isInsecure()) {
+	if (hashAlgo->count() > 0 && hashAlgo->current().isInsecure()) {
 		gotoTab(0);
-		xcaWarning msg(this, tr("The currently selected hash algorithm '%1' is insecure and should not be used.").arg(hashAlgo->currentHashName()));
+		xcaWarningBox msg(this, tr("The currently selected hash algorithm '%1' is insecure and should not be used.").arg(hashAlgo->current().name()));
 		msg.addButton(QMessageBox::Ok, tr("Select other algorithm"));
 		msg.addButton(QMessageBox::Yes, tr("Use algorithm anyway"));
 		if (msg.exec() == QMessageBox::Ok)
@@ -1297,7 +1313,7 @@ void NewX509::accept()
 	if (signer && notBefore->getDate() < signer->getNotBefore()) {
 		gotoTab(2);
 		QString text = tr("The certificate will be earlier valid than the signer. This is probably not what you want.");
-		xcaWarning msg(this, text);
+		xcaWarningBox msg(this, text);
 		msg.addButton(QMessageBox::Ok, tr("Edit dates"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
@@ -1320,7 +1336,7 @@ void NewX509::accept()
 				!noWellDefinedExpDate->isChecked()) {
 		gotoTab(2);
 		QString text = tr("The certificate will be longer valid than the signer. This is probably not what you want.");
-		xcaWarning msg(this, text);
+		xcaWarningBox msg(this, text);
 		msg.addButton(QMessageBox::Ok, tr("Edit dates"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
@@ -1343,7 +1359,7 @@ void NewX509::accept()
 	    notBefore->getDate() > notAfter->getDate()) {
 		gotoTab(2);
 		QString text = tr("The certificate will be out of date before it becomes valid. You most probably mixed up both dates.");
-		xcaWarning msg(this, text);
+		xcaWarningBox msg(this, text);
 		msg.addButton(QMessageBox::Ok, tr("Edit dates"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
@@ -1362,14 +1378,21 @@ void NewX509::accept()
 	int r = do_validateExtensions();
 	if (r) {
 		QString text;
-		if (r == 1) {
+		switch (r) {
+		case 1:
 			text = tr("The certificate contains invalid or duplicate extensions. Check the validation on the advanced tab.");
 			gotoTab(5);
-		} else {
+			break;
+		case 2:
 			text = tr("The certificate contains no extensions. You may apply the extensions of one of the templates to define the purpose of the certificate.");
 			gotoTab(0);
+			break;
+		case 3:
+			text = tr("The certificate contains invalid extensions.");
+			gotoTab(2);
+			break;
 		}
-		xcaWarning msg(this, text);
+		xcaWarningBox msg(this, text);
 		msg.addButton(QMessageBox::Ok, tr("Edit extensions"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
@@ -1386,10 +1409,10 @@ void NewX509::accept()
 		}
 	}
 	QString cn = xn.getEntryByNid(NID_commonName);
-	QStringList san = subAltName->text().split(QRegExp(" *, *"));
+	QStringList san = subAltName->text().split(QRegularExpression(" *, *"));
 	if (cn.isEmpty() && san.contains("DNS:copycn") && pt != tmpl) {
 		gotoTab(2);
-		xcaWarning msg(this, tr("The subject alternative name shall contain a copy of the common name. However, the common name is empty."));
+		xcaWarningBox msg(this, tr("The subject alternative name shall contain a copy of the common name. However, the common name is empty."));
 		msg.addButton(QMessageBox::Ok, tr("Edit extensions"));
 		msg.addButton(QMessageBox::Close, tr("Abort rollout"));
 		msg.addButton(QMessageBox::Apply, tr("Continue rollout"));
