@@ -19,8 +19,6 @@
 #include "lib/BioByteArray.h"
 #include "lib/func.h"
 
-#include "lib/openssl_compat.h"
-
 x509v3ext NewX509::getBasicConstraints()
 {
 	QStringList cont;
@@ -55,11 +53,7 @@ x509v3ext NewX509::getOCSPstaple()
 	x509v3ext ext;
 	if (OCSPstaple->isChecked())
 		ext.create(NID_tlsfeature,
-#ifdef NID_tlsfeature
 			"status_request",
-#else
-			"DER:30:03:02:01:05",
-#endif
 			&ext_ctx);
 	return ext;
 }
@@ -84,7 +78,6 @@ x509v3ext NewX509::getKeyUsage()
 	};
 
 	QStringList cont;
-	x509v3ext ext;
 
 	int rows = keyUsage->count();
 	for (int i=0; i<rows; i++) {
@@ -93,14 +86,13 @@ x509v3ext NewX509::getKeyUsage()
 	}
 	if (kuCritical->isChecked() && cont.count() > 0)
 		cont.prepend("critical");
-	ext.create(NID_key_usage, cont.join(", "), &ext_ctx);
-	return ext;
+
+	return x509v3ext(NID_key_usage, cont.join(", "), &ext_ctx);
 }
 
 x509v3ext NewX509::getEkeyUsage()
 {
 	QStringList cont;
-	x509v3ext ext;
 
 	int rows = ekeyUsage->count();
 	for (int i=0; i<rows; i++) {
@@ -109,13 +101,12 @@ x509v3ext NewX509::getEkeyUsage()
 	}
 	if (ekuCritical->isChecked() && cont.count() > 0)
 		cont.prepend("critical");
-	ext.create(NID_ext_key_usage, cont.join(", "), &ext_ctx);
-	return ext;
+
+	return x509v3ext(NID_ext_key_usage, cont.join(", "), &ext_ctx);
 }
 
 x509v3ext NewX509::getSubAltName()
 {
-	x509v3ext ext;
 	QString s = subAltName->text();
 	if (pt == x509_req) {
 		QStringList sn, sl = s.split(',');
@@ -125,13 +116,11 @@ x509v3ext NewX509::getSubAltName()
 		}
 		s = sn.join(",");
 	}
-	ext.create(NID_subject_alt_name, s, &ext_ctx);
-	return ext;
+	return x509v3ext(NID_subject_alt_name, s, &ext_ctx);
 }
 
 x509v3ext NewX509::getIssAltName()
 {
-	x509v3ext ext;
 	QString s = issAltName->text();
 	if (pt == x509_req) {
 		QStringList sn, sl = s.split(',');
@@ -141,26 +130,17 @@ x509v3ext NewX509::getIssAltName()
 		}
 		s = sn.join(",");
 	}
-	ext.create(NID_issuer_alt_name, s, &ext_ctx);
-	return ext;
+	return x509v3ext(NID_issuer_alt_name, s, &ext_ctx);
 }
 
 x509v3ext NewX509::getCrlDist()
 {
-	x509v3ext ext;
-	if (!crlDist->text().isEmpty()) {
-		ext.create(NID_crl_distribution_points, crlDist->text(), &ext_ctx);
-	}
-	return ext;
+	return x509v3ext(NID_crl_distribution_points, crlDist->text(), &ext_ctx);
 }
 
 x509v3ext NewX509::getAuthInfAcc()
 {
-	x509v3ext ext;
-	if (!authInfAcc->text().isEmpty()) {
-		ext.create(NID_info_access, authInfAcc->text(), &ext_ctx);
-	}
-	return ext;
+	return x509v3ext(NID_info_access, authInfAcc->text(), &ext_ctx);
 }
 
 extList NewX509::getAdvanced()
@@ -186,7 +166,7 @@ extList NewX509::getAdvanced()
 	ret = NCONF_load_bio(conf, BioByteArray(conf_str.toLatin1()).ro(),
 				&err_line);
 	if (ret != 1) {
-		openssl_error(tr("Configfile error on line %1\n").
+		openssl_error_msg(tr("Configfile error on line %1\n").
 				arg(err_line));
 		return elist;
 	}
@@ -247,6 +227,7 @@ extList NewX509::getAllExt()
 	ne += getAdvanced();
 	if (!Settings["disable_netscape"])
 		ne += getNetscapeExt();
+	ne.delInvalid();
 	return ne;
 }
 
@@ -256,7 +237,6 @@ extList NewX509::getNetscapeExt()
 		"client", "server",  "email", "objsign",
 		"sslCA",  "emailCA", "objCA" };
 
-
 	QStringList cont;
 	x509v3ext ext;
 	extList el;
@@ -267,7 +247,7 @@ extList NewX509::getNetscapeExt()
 			cont << certTypeList[i];
 	}
 
-	el << ext.create(NID_netscape_cert_type, cont.join(", "), &ext_ctx);
+	el << x509v3ext(NID_netscape_cert_type, cont.join(", "), &ext_ctx);
 	el << ext.create_ia5(NID_netscape_base_url, nsBaseUrl->text(), &ext_ctx);
 	el << ext.create_ia5(NID_netscape_revocation_url, nsRevocationUrl->text(), &ext_ctx);
 	el << ext.create_ia5(NID_netscape_ca_revocation_url, nsCARevocationUrl->text(), &ext_ctx);
