@@ -6,6 +6,7 @@
  */
 
 #include "lib/func.h"
+#include "lib/load_obj.h"
 #include "Options.h"
 #include "SearchPkcs11.h"
 #include "XcaWarning.h"
@@ -37,7 +38,8 @@ Options::Options(QWidget *parent)
 	  << tr("UTF8 strings only (RFC2459)")
 	  << tr("All strings");
 	mbstring->addItems(s);
-	mbstring->setCurrentIndex(string_opts.indexOf(Settings["string_opt"]));
+	mbstring->setCurrentIndex(string_opts.indexOf(
+								QString(Settings["string_opt"])));
 
 	searchP11 = NULL;
 	transDnEntries->setText(transDnEntries->text()
@@ -144,7 +146,8 @@ int Options::exec()
 	Settings["only_token_hashes"] = onlyTokenHashes->checkState();
 	Settings["disable_netscape"] = disableNetscape->checkState();
 
-	Settings["default_hash"] = hashAlgo->currentHashName();
+	Settings["default_hash"] = hashAlgo->current().name();
+    	Settings["pkcs12_enc_algo"] = pkcs12EncAlgo->current().name();
 	Settings["mandatory_dn"] = getDnString(extDNlist);
 	Settings["explicit_dn"] = getDnString(expDNlist);
 	Settings["string_opt"] = string_opts[mbstring->currentIndex()];
