@@ -28,12 +28,14 @@ class TempTreeView: public XcaTreeView
 			const QModelIndex &index, QModelIndexList indexes);
 	void showPki(pki_base *pki);
 	bool alterTemp(pki_temp *temp);
+	ExportDialog *exportDialog(const QModelIndexList &index);
 
    public slots:
 	void certFromTemp();
 	void reqFromTemp();
 	void duplicateTemp();
 	void newItem();
+	void load();
 
     signals:
 	void newReq(pki_temp *);
